@@ -18,38 +18,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "config.h"
+#if HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "libscdl.h"
 
-#ifdef HAVE_LTDL_H
-#include <ltdl.h>
-/* libltdl is present, pass all calls to it */
-
-void *sc_dlopen(const char *filename)
-{
-	return (void *)lt_dlopen(filename);
-}
-
-void *sc_dlsym(void *handle, const char *symbol)
-{
-	return lt_dlsym((lt_dlhandle)handle, symbol);
-}
-
-const char *sc_dlerror(void)
-{
-	return lt_dlerror();
-}
-
-int sc_dlclose(void *handle)
-{
-	return lt_dlclose((lt_dlhandle)handle);
-}
-
-#else
-/* Small wrappers for native functions, bypassing libltdl */
-#ifdef _WIN32
-/* Use Windows calls */
+#ifdef WIN32
+#include <windows.h>
 void *sc_dlopen(const char *filename)
 {
 	return (void *)LoadLibrary(filename);
@@ -57,7 +33,7 @@ void *sc_dlopen(const char *filename)
 
 void *sc_dlsym(void *handle, const char *symbol)
 {
-	return GetProcAddress(handle, symbol);
+	return GetProcAddress((HANDLE)handle, symbol);
 }
 
 const char *sc_dlerror()
@@ -67,12 +43,10 @@ const char *sc_dlerror()
 
 int sc_dlclose(void *handle)
 {
-	return FreeLibrary(handle);
+	return FreeLibrary((HANDLE)handle);
 }
-
-#elif defined(HAVE_DLFCN_H)
+#else
 #include <dlfcn.h>
-/* Use native interfaces */
 void *sc_dlopen(const char *filename)
 {
 	return dlopen(filename, RTLD_LAZY);
@@ -92,28 +66,4 @@ int sc_dlclose(void *handle)
 {
 	return dlclose(handle);
 }
-
-#else
-/* Dynamic loading is not available */
-void *sc_dlopen(const char *filename)
-{
-	return NULL;
-}
-
-void *sc_dlsym(void *handle, const char *symbol)
-{
-	return NULL;
-}
-
-const char *sc_dlerror()
-{
-	return "dlopen() functionality not available";
-}
-
-int sc_dlclose(void *handle)
-{
-	return 0;
-}
-
-#endif
 #endif
