@@ -455,6 +455,8 @@ static int entersafe_select_fid(sc_card_t *card,
 	sc_file_t *file=0;
 	sc_path_t path;
 
+	memset(&path, 0, sizeof(sc_path_t));
+
 	path.type=SC_PATH_TYPE_FILE_ID;
 	path.value[0]=id_hi;
 	path.value[1]=id_lo;
@@ -489,7 +491,7 @@ static int entersafe_select_aid(sc_card_t *card,
 {
 	int r = 0;
 
-	if (card->cache_valid 
+	if (card->cache.valid 
 		&& card->cache.current_path.type == SC_PATH_TYPE_DF_NAME
 		&& card->cache.current_path.len == in_path->len
 		&& memcmp(card->cache.current_path.value, in_path->value, in_path->len)==0 )
@@ -557,7 +559,7 @@ static int entersafe_select_path(sc_card_t *card,
 	 }
 	
 	 /* check current working directory */
-	 if (card->cache_valid 
+	 if (card->cache.valid 
 		 && card->cache.current_path.type == SC_PATH_TYPE_PATH
 		 && card->cache.current_path.len >= 2
 		 && card->cache.current_path.len <= pathlen )
@@ -569,7 +571,7 @@ static int entersafe_select_path(sc_card_t *card,
 					bMatch += 2;
 	 }
 
-	 if ( card->cache_valid && bMatch > 2 )
+	 if ( card->cache.valid && bMatch > 2 )
 	 {
 		  if ( pathlen - bMatch == 2 )
 		  {
@@ -584,7 +586,9 @@ static int entersafe_select_path(sc_card_t *card,
 			   /* first step: change directory */
 			   r = entersafe_select_fid(card, path[bMatch], path[bMatch+1], NULL);
 			   SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "SELECT FILE (DF-ID) failed");
-		
+	
+		   	   memset(&new_path, 0, sizeof(sc_path_t));
+
 			   new_path.type = SC_PATH_TYPE_PATH;
 			   new_path.len  = pathlen - bMatch-2;
 			   memcpy(new_path.value, &(path[bMatch+2]), new_path.len);
@@ -646,7 +650,7 @@ static int entersafe_select_file(sc_card_t *card,
 	  sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
 		"current path (%s, %s): %s (len: %u)\n",
 		   (card->cache.current_path.type==SC_PATH_TYPE_DF_NAME?"aid":"path"),
-		   (card->cache_valid?"valid":"invalid"), pbuf,
+		   (card->cache.valid?"valid":"invalid"), pbuf,
 		   card->cache.current_path.len);
 	 
 	 switch(in_path->type)
@@ -952,6 +956,7 @@ static int entersafe_pin_cmd(sc_card_t *card, struct sc_pin_cmd_data *data,
 	 if(data->cmd!=SC_PIN_CMD_UNBLOCK)
 	 {
 		  r = iso_ops->pin_cmd(card,data,tries_left);
+		  sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Verify rv:%i", r);
 	 }
 	 else
 	 {
@@ -1005,7 +1010,7 @@ static int entersafe_erase_card(sc_card_t *card)
 	r = entersafe_transmit_apdu(card, &apdu,0,0,0,0);
 	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	/* invalidate cache */
-	card->cache_valid = 0;
+	card->cache.valid = 0;
 
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0xEE, 0x00, 0x00);
 	apdu.cla=0x84;
