@@ -18,11 +18,15 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
+#include "config.h"
+
+#include <stdlib.h>
+#include <string.h>
+
 #include "internal.h"
 #include "asn1.h"
 #include "cardctl.h"
-#include <stdlib.h>
-#include <string.h>
 
 static struct sc_atr_table starcos_atrs[] = {
 	{ "3B:B7:94:00:c0:24:31:fe:65:53:50:4b:32:33:90:00:b4", NULL, NULL, SC_CARD_TYPE_STARCOS_GENERIC, 0, NULL },
@@ -51,7 +55,7 @@ static const struct sc_card_error starcos_errors[] =
 	{ 0x6F01, SC_ERROR_CARD_CMD_FAILED, "public key not complete"},
 	{ 0x6F02, SC_ERROR_CARD_CMD_FAILED, "data overflow"},
 	{ 0x6F03, SC_ERROR_CARD_CMD_FAILED, "invalid command sequence"},
-	{ 0x6F05, SC_ERROR_CARD_CMD_FAILED, "security enviroment invalid"},
+	{ 0x6F05, SC_ERROR_CARD_CMD_FAILED, "security environment invalid"},
 	{ 0x6F07, SC_ERROR_FILE_NOT_FOUND, "key part not found"},
 	{ 0x6F08, SC_ERROR_CARD_CMD_FAILED, "signature failed"},
 	{ 0x6F0A, SC_ERROR_INCORRECT_PARAMETERS, "key format does not match key length"},
@@ -59,7 +63,7 @@ static const struct sc_card_error starcos_errors[] =
 	{ 0x6F81, SC_ERROR_CARD_CMD_FAILED, "system error"}
 };
 
-/* internal structure to save the current security enviroment */
+/* internal structure to save the current security environment */
 typedef struct starcos_ex_data_st {
 	int    sec_ops;	/* the currently selected security operation,
 			 * i.e. SC_SEC_OPERATION_AUTHENTICATE etc. */
@@ -82,7 +86,7 @@ static int starcos_init(sc_card_t *card)
 	unsigned int flags;
 	starcos_ex_data *ex_data;
 
-	ex_data = (starcos_ex_data *) calloc(1, sizeof(starcos_ex_data));
+	ex_data = calloc(1, sizeof(starcos_ex_data));
 	if (ex_data == NULL)
 		return SC_ERROR_OUT_OF_MEMORY;
 
@@ -92,7 +96,6 @@ static int starcos_init(sc_card_t *card)
 
 	flags = SC_ALGORITHM_RSA_PAD_PKCS1 
 		| SC_ALGORITHM_ONBOARD_KEY_GEN
-		| SC_CARD_FLAG_RNG
 		| SC_ALGORITHM_RSA_PAD_ISO9796
 		| SC_ALGORITHM_RSA_HASH_NONE
 		| SC_ALGORITHM_RSA_HASH_SHA1
@@ -107,10 +110,8 @@ static int starcos_init(sc_card_t *card)
 	card->caps = SC_CARD_CAP_RNG; 
 
 	/* we need read_binary&friends with max 128 bytes per read */
-	if (card->max_send_size > 128)
-		card->max_send_size = 128;
-	if (card->max_recv_size > 128)
-		card->max_recv_size = 128;
+	card->max_send_size = 128;
+	card->max_recv_size = 128;
 
 	return 0;
 }
@@ -134,8 +135,7 @@ static int process_fci(sc_context_t *ctx, sc_file_t *file,
 	size_t taglen, len = buflen;
 	const u8 *tag = NULL, *p;
   
-	if (ctx->debug >= 3)
-		sc_debug(ctx, "processing FCI bytes\n");
+	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "processing FCI bytes\n");
 
 	if (buflen < 2)
 		return SC_ERROR_INTERNAL;
@@ -156,8 +156,8 @@ static int process_fci(sc_context_t *ctx, sc_file_t *file,
 	tag = sc_asn1_find_tag(ctx, p, len, 0x80, &taglen);
 	if (tag != NULL && taglen >= 2) {
 		int bytes = (tag[0] << 8) + tag[1];
-		if (ctx->debug >= 3)
-			sc_debug(ctx, "  bytes in file: %d\n", bytes);
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+			"  bytes in file: %d\n", bytes);
 		file->size = bytes;
 	}
 
@@ -205,10 +205,10 @@ static int process_fci(sc_context_t *ctx, sc_file_t *file,
 			}
 		}
 
-		if (ctx->debug >= 3) {
-	 		sc_debug(ctx, "  type: %s\n", type);
-			sc_debug(ctx, "  EF structure: %s\n", structure);
-		}
+ 		sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+			"  type: %s\n", type);
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+			"  EF structure: %s\n", structure);
 	}
 	file->magic = SC_FILE_MAGIC;
 
@@ -230,11 +230,11 @@ static int starcos_select_aid(sc_card_t *card,
 	apdu.resplen = 0;
 	apdu.le = 0;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 
 	/* check return value */
 	if (!(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) && apdu.sw1 != 0x61 )
-    		SC_FUNC_RETURN(card->ctx, 2, sc_check_sw(card, apdu.sw1, apdu.sw2));
+    		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
   
 	/* update cache */
 	card->cache.current_path.type = SC_PATH_TYPE_DF_NAME;
@@ -244,7 +244,7 @@ static int starcos_select_aid(sc_card_t *card,
 	if (file_out) {
 		sc_file_t *file = sc_file_new();
 		if (!file)
-			SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_OUT_OF_MEMORY);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_OUT_OF_MEMORY);
 		file->type = SC_FILE_TYPE_DF;
 		file->ef_structure = SC_FILE_EF_UNKNOWN;
 		file->path.len = 0;
@@ -257,7 +257,7 @@ static int starcos_select_aid(sc_card_t *card,
 		file->magic = SC_FILE_MAGIC;
 		*file_out = file;
 	}
-	SC_FUNC_RETURN(card->ctx, 2, SC_SUCCESS);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_SUCCESS);
 }
 
 static int starcos_select_fid(sc_card_t *card,
@@ -280,7 +280,7 @@ static int starcos_select_fid(sc_card_t *card,
 	apdu.datalen = 2;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 
 	if (apdu.p2 == 0x00 && apdu.sw1 == 0x62 && apdu.sw2 == 0x84 ) {
 		/* no FCI => we have a DF (see comment in process_fci()) */
@@ -290,7 +290,7 @@ static int starcos_select_fid(sc_card_t *card,
 		apdu.resplen = 0;
 		apdu.le = 0;
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, r, "APDU re-transmit failed");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU re-transmit failed");
     	} else if (apdu.sw1 == 0x61 || (apdu.sw1 == 0x90 && apdu.sw2 == 0x00)) {
 		/* SELECT returned some data (possible FCI) =>
 		 * try a READ BINARY to see if a EF is selected */
@@ -302,14 +302,14 @@ static int starcos_select_fid(sc_card_t *card,
 		apdu2.le = 1;
 		apdu2.lc = 0;
 		r = sc_transmit_apdu(card, &apdu2);
-		SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 		if (apdu2.sw1 == 0x69 && apdu2.sw2 == 0x86)
 			/* no current EF is selected => we have a DF */
 			bIsDF = 1;
 	}
 
 	if (apdu.sw1 != 0x61 && (apdu.sw1 != 0x90 || apdu.sw2 != 0x00))
-		SC_FUNC_RETURN(card->ctx, 2, sc_check_sw(card, apdu.sw1, apdu.sw2));
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
 
 	/* update cache */
 	if (bIsDF) {
@@ -328,7 +328,7 @@ static int starcos_select_fid(sc_card_t *card,
 	if (file_out) {
 		sc_file_t *file = sc_file_new();
 		if (!file)
-			SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_OUT_OF_MEMORY);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_OUT_OF_MEMORY);
 		file->id   = (id_hi << 8) + id_lo;
 		file->path = card->cache.current_path;
 
@@ -353,7 +353,7 @@ static int starcos_select_fid(sc_card_t *card,
 		}
 	}
 
-	SC_FUNC_RETURN(card->ctx, 2, SC_SUCCESS);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_SUCCESS);
 }
 
 static int starcos_select_file(sc_card_t *card,
@@ -363,21 +363,19 @@ static int starcos_select_file(sc_card_t *card,
 	u8 pathbuf[SC_MAX_PATH_SIZE], *path = pathbuf;
 	int    r;
 	size_t i, pathlen;
+	char pbuf[SC_MAX_PATH_STRING_SIZE];
 
-	SC_FUNC_CALLED(card->ctx, 1);
+	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
 
-	if (card->ctx->debug >= 4) {
-		char pbuf[SC_MAX_PATH_STRING_SIZE];
+	r = sc_path_print(pbuf, sizeof(pbuf), &card->cache.current_path);
+	if (r != SC_SUCCESS)
+		pbuf[0] = '\0';
 
-		r = sc_path_print(pbuf, sizeof(pbuf), &card->cache.current_path);
-		if (r != SC_SUCCESS)
-			pbuf[0] = '\0';
-
-		sc_debug(card->ctx, "current path (%s, %s): %s (len: %u)\n",
-			(card->cache.current_path.type==SC_PATH_TYPE_DF_NAME?"aid":"path"),
-			(card->cache_valid?"valid":"invalid"), pbuf,
-			card->cache.current_path.len);
-	}
+	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+		"current path (%s, %s): %s (len: %u)\n",
+		(card->cache.current_path.type==SC_PATH_TYPE_DF_NAME?"aid":"path"),
+		(card->cache.valid?"valid":"invalid"), pbuf,
+		card->cache.current_path.len);
   
 	memcpy(path, in_path->value, in_path->len);
 	pathlen = in_path->len;
@@ -386,20 +384,19 @@ static int starcos_select_file(sc_card_t *card,
 	{	/* SELECT EF/DF with ID */
 		/* Select with 2byte File-ID */
 		if (pathlen != 2)
-			SC_FUNC_RETURN(card->ctx,2,SC_ERROR_INVALID_ARGUMENTS);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE,SC_ERROR_INVALID_ARGUMENTS);
 		return starcos_select_fid(card, path[0], path[1], file_out);
 	}
 	else if (in_path->type == SC_PATH_TYPE_DF_NAME)
       	{	/* SELECT DF with AID */
 		/* Select with 1-16byte Application-ID */
-		if (card->cache_valid 
+		if (card->cache.valid 
 		    && card->cache.current_path.type == SC_PATH_TYPE_DF_NAME
 		    && card->cache.current_path.len == pathlen
 		    && memcmp(card->cache.current_path.value, pathbuf, pathlen) == 0 )
 		{
-			if (card->ctx->debug >= 4)
-				sc_debug(card->ctx, "cache hit\n");
-			SC_FUNC_RETURN(card->ctx, 2, SC_SUCCESS);
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "cache hit\n");
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_SUCCESS);
 		}
 		else
 			return starcos_select_aid(card, pathbuf, pathlen, file_out);
@@ -416,10 +413,10 @@ static int starcos_select_file(sc_card_t *card,
 		 * of a EF) => pathlen must be even and less than 6
 		 */
 		if (pathlen%2 != 0 || pathlen > 6 || pathlen <= 0)
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_INVALID_ARGUMENTS);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
 		/* if pathlen == 6 then the first FID must be MF (== 3F00) */
 		if (pathlen == 6 && ( path[0] != 0x3f || path[1] != 0x00 ))
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_INVALID_ARGUMENTS);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
 
 		/* unify path (the first FID should be MF) */
 		if (path[0] != 0x3f || path[1] != 0x00)
@@ -433,7 +430,7 @@ static int starcos_select_file(sc_card_t *card,
 		}
 	
 		/* check current working directory */
-		if (card->cache_valid 
+		if (card->cache.valid 
 		    && card->cache.current_path.type == SC_PATH_TYPE_PATH
 		    && card->cache.current_path.len >= 2
 		    && card->cache.current_path.len <= pathlen )
@@ -445,7 +442,7 @@ static int starcos_select_file(sc_card_t *card,
 					bMatch += 2;
 		}
 
-		if ( card->cache_valid && bMatch >= 0 )
+		if ( card->cache.valid && bMatch >= 0 )
 		{
 			if ( pathlen - bMatch == 2 )
 				/* we are in the rigth directory */
@@ -457,8 +454,9 @@ static int starcos_select_file(sc_card_t *card,
 	
 				/* first step: change directory */
 				r = starcos_select_fid(card, path[bMatch], path[bMatch+1], NULL);
-				SC_TEST_RET(card->ctx, r, "SELECT FILE (DF-ID) failed");
-		
+				SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "SELECT FILE (DF-ID) failed");
+	
+				memset(&new_path, 0, sizeof(sc_path_t));	
 				new_path.type = SC_PATH_TYPE_PATH;
 				new_path.len  = pathlen - bMatch-2;
 				memcpy(new_path.value, &(path[bMatch+2]), new_path.len);
@@ -469,13 +467,13 @@ static int starcos_select_file(sc_card_t *card,
 			{
 				/* done: we are already in the
 				 * requested directory */
-				if ( card->ctx->debug >= 4 )
-					sc_debug(card->ctx, "cache hit\n");
+				sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+					"cache hit\n");
 				/* copy file info (if necessary) */
 				if (file_out) {
 					sc_file_t *file = sc_file_new();
 					if (!file)
-						SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_OUT_OF_MEMORY);
+						SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_OUT_OF_MEMORY);
 					file->id = (path[pathlen-2] << 8) +
 						   path[pathlen-1];
 					file->path = card->cache.current_path;
@@ -496,13 +494,13 @@ static int starcos_select_file(sc_card_t *card,
 			for ( i=0; i<pathlen-2; i+=2 )
 			{
 				r = starcos_select_fid(card, path[i], path[i+1], NULL);
-				SC_TEST_RET(card->ctx, r, "SELECT FILE (DF-ID) failed");
+				SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "SELECT FILE (DF-ID) failed");
 			}
 			return starcos_select_fid(card, path[pathlen-2], path[pathlen-1], file_out);
 		}
 	}
 	else
-		SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_INVALID_ARGUMENTS);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
 }
 
 #define STARCOS_AC_ALWAYS	0x9f
@@ -570,7 +568,7 @@ static int starcos_process_acl(sc_card_t *card, sc_file_t *file,
 			tmp = 0x00;	/* no sm */
 		*p++ = tmp;	/* use the same sm mode for all ops */
 		*p++ = tmp;
-		*p++ = tmp;
+		*p = tmp;
 		data->type = SC_STARCOS_MF_DATA;
 
 		return SC_SUCCESS;
@@ -607,7 +605,7 @@ static int starcos_process_acl(sc_card_t *card, sc_file_t *file,
 		else
 			tmp = 0x00;
 		*p++ = tmp;	/* SM CR  */
-		*p++ = tmp;	/* SM ISF */
+		*p = tmp;	/* SM ISF */
 
 		data->data.df.size[0] = (file->size >> 8) & 0xff;
 		data->data.df.size[1] = file->size & 0xff;
@@ -645,17 +643,17 @@ static int starcos_process_acl(sc_card_t *card, sc_file_t *file,
 		case SC_FILE_EF_TRANSPARENT:
 			*p++ = 0x81;
 			*p++ = (file->size >> 8) & 0xff;
-			*p++ = file->size & 0xff;
+			*p = file->size & 0xff;
 			break;
 		case SC_FILE_EF_LINEAR_FIXED:
 			*p++ = 0x82;
 			*p++ = file->record_count  & 0xff;
-			*p++ = file->record_length & 0xff;
+			*p = file->record_length & 0xff;
 			break;
 		case SC_FILE_EF_CYCLIC:
 			*p++ = 0x84;
 			*p++ = file->record_count  & 0xff;
-			*p++ = file->record_length & 0xff;
+			*p = file->record_length & 0xff;
 			break;
 		default:
 			return SC_ERROR_INVALID_ARGUMENTS;
@@ -683,8 +681,7 @@ static int starcos_create_mf(sc_card_t *card, sc_starcos_create_data *data)
 	sc_apdu_t       apdu;
 	sc_context_t   *ctx = card->ctx;
 
-	if (ctx->debug >= 3)
-		sc_debug(ctx, "creating MF \n");
+	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "creating MF \n");
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0xE0, 0x00, 0x00);
 	apdu.cla |= 0x80;
 	apdu.lc   = 19;
@@ -692,7 +689,7 @@ static int starcos_create_mf(sc_card_t *card, sc_starcos_create_data *data)
 	apdu.data = (u8 *) data->data.mf.header;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(ctx, r, "APDU transmit failed");
+	SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	return sc_check_sw(card, apdu.sw1, apdu.sw2);	
 }
 
@@ -713,11 +710,9 @@ static int starcos_create_df(sc_card_t *card, sc_starcos_create_data *data)
 	sc_apdu_t       apdu;
 	sc_context_t   *ctx = card->ctx;
 
-	if (ctx->debug >= 3)
-		sc_debug(ctx, "creating DF\n");
+	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "creating DF\n");
 	/* first step: REGISTER DF */
-	if (ctx->debug >= 3)
-		sc_debug(ctx, "calling REGISTER DF\n");
+	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "calling REGISTER DF\n");
 
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0x52,
 		       data->data.df.size[0], data->data.df.size[1]);
@@ -728,10 +723,9 @@ static int starcos_create_df(sc_card_t *card, sc_starcos_create_data *data)
 	apdu.data = data->data.df.header;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(ctx, r, "APDU transmit failed");
+	SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	/* second step: CREATE DF */
-	if (ctx->debug >= 3)
-		sc_debug(ctx, "calling CREATE DF\n");
+	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "calling CREATE DF\n");
 
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0xE0, 0x01, 0x00);
 	apdu.cla |= 0x80;
@@ -740,7 +734,7 @@ static int starcos_create_df(sc_card_t *card, sc_starcos_create_data *data)
 	apdu.data = data->data.df.header;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(ctx, r, "APDU transmit failed");
+	SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	return sc_check_sw(card, apdu.sw1, apdu.sw2);
 }
 
@@ -759,8 +753,7 @@ static int starcos_create_ef(sc_card_t *card, sc_starcos_create_data *data)
 	sc_apdu_t       apdu;
 	sc_context_t   *ctx = card->ctx;
 
-	if (ctx->debug >= 3)
-		sc_debug(ctx, "creating EF\n");
+	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "creating EF\n");
 
 	sc_format_apdu(card,&apdu,SC_APDU_CASE_3_SHORT,0xE0,0x03,0x00);
 	apdu.cla |= 0x80;
@@ -769,7 +762,7 @@ static int starcos_create_ef(sc_card_t *card, sc_starcos_create_data *data)
 	apdu.data = (u8 *) data->data.ef.header;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	return sc_check_sw(card, apdu.sw1, apdu.sw2);
 }
 
@@ -799,7 +792,7 @@ static int starcos_create_end(sc_card_t *card, sc_file_t *file)
 	apdu.datalen = 2;
 	apdu.data = fid;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	return sc_check_sw(card, apdu.sw1, apdu.sw2);
 }
 
@@ -816,7 +809,7 @@ static int starcos_create_file(sc_card_t *card, sc_file_t *file)
 	int    r;
 	sc_starcos_create_data data;
 
-	SC_FUNC_CALLED(card->ctx, 1);
+	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
 
 	if (file->type == SC_FILE_TYPE_DF) {
 		if (file->id == 0x3f00) {
@@ -864,9 +857,9 @@ static int starcos_erase_card(sc_card_t *card)
 	apdu.data = sbuf;
 	
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	/* invalidate cache */
-	card->cache_valid = 0;
+	card->cache.valid = 0;
 	if (apdu.sw1 == 0x69 && apdu.sw2 == 0x85)
 		/* no MF to delete, ignore error */
 		return SC_SUCCESS;
@@ -906,7 +899,7 @@ static int starcos_write_key(sc_card_t *card, sc_starcos_wkey_data *data)
 		apdu.data = sbuf;
 
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 		if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
 			return sc_check_sw(card, apdu.sw1, apdu.sw2);
 		if (data->key == NULL)
@@ -936,7 +929,7 @@ static int starcos_write_key(sc_card_t *card, sc_starcos_wkey_data *data)
 		apdu.data    = sbuf;
 
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 		if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
 			return sc_check_sw(card, apdu.sw1, apdu.sw2);
 		offset += clen;
@@ -972,7 +965,7 @@ static int starcos_gen_key(sc_card_t *card, sc_starcos_gen_key_data *data)
 	apdu.lc      = 2;
 	apdu.datalen = 2;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
 		return sc_check_sw(card, apdu.sw1, apdu.sw2);
 	/* read public key via READ PUBLIC KEY */
@@ -986,11 +979,11 @@ static int starcos_gen_key(sc_card_t *card, sc_starcos_gen_key_data *data)
 	apdu.resplen = sizeof(rbuf);
 	apdu.le      = 256;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
 		return sc_check_sw(card, apdu.sw1, apdu.sw2);
 
-	data->modulus = (u8 *) malloc(len);
+	data->modulus = malloc(len);
 	if (!data->modulus)
 		return SC_ERROR_OUT_OF_MEMORY;
 	p = data->modulus;
@@ -1004,14 +997,14 @@ static int starcos_gen_key(sc_card_t *card, sc_starcos_gen_key_data *data)
 }
 
 /** starcos_set_security_env
- * sets the security enviroment
+ * sets the security environment
  * \param card pointer to the sc_card object
  * \param env pointer to a sc_security_env object
  * \param se_num not used here
  * \return SC_SUCCESS on success or an error code
  *
- * This function sets the security enviroment (using the starcos spk 2.3
- * command MANAGE SECURITY ENVIROMENT). In case a COMPUTE SIGNATURE
+ * This function sets the security environment (using the starcos spk 2.3
+ * command MANAGE SECURITY ENVIRONMENT). In case a COMPUTE SIGNATURE
  * operation is requested , this function tries to detect whether
  * COMPUTE SIGNATURE or INTERNAL AUTHENTICATE must be used for signature
  * calculation.
@@ -1020,14 +1013,13 @@ static int starcos_set_security_env(sc_card_t *card,
 				    const sc_security_env_t *env,
 				    int se_num)
 {
-	u8              *p, *pp, keyID;
+	u8              *p, *pp;
 	int              r, operation = env->operation;
 	sc_apdu_t   apdu;
 	u8               sbuf[SC_MAX_APDU_BUFFER_SIZE];
 	starcos_ex_data *ex_data = (starcos_ex_data *)card->drv_data;
 
 	p     = sbuf;
-	keyID = env->key_ref[0];
 
 	/* copy key reference, if present */
 	if (env->flags & SC_SEC_ENV_KEY_REF_PRESENT) {
@@ -1054,9 +1046,9 @@ static int starcos_set_security_env(sc_card_t *card,
 		apdu.lc      = p - sbuf;
 		apdu.le      = 0;
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 		if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
-			SC_FUNC_RETURN(card->ctx, 4, sc_check_sw(card, apdu.sw1, apdu.sw2));
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
 		return SC_SUCCESS;
 	}
 	/* try COMPUTE SIGNATURE */
@@ -1101,12 +1093,10 @@ static int starcos_set_security_env(sc_card_t *card,
 		apdu.datalen = p - sbuf;
 		apdu.lc      = p - sbuf;
 		apdu.le      = 0;
-		/* suppress errors, as don't know whether to use 
+		/* we don't know whether to use 
 		 * COMPUTE SIGNATURE or INTERNAL AUTHENTICATE */
-		sc_ctx_suppress_errors_on(card->ctx);
 		r = sc_transmit_apdu(card, &apdu);
-		sc_ctx_suppress_errors_off(card->ctx);
-		SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 		if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 			ex_data->fix_digestInfo = 0;
 			ex_data->sec_ops        = SC_SEC_OPERATION_SIGN;
@@ -1131,9 +1121,9 @@ try_authenticate:
 		apdu.lc      = p - sbuf;
 		apdu.le      = 0;
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 		if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
-			SC_FUNC_RETURN(card->ctx, 4, sc_check_sw(card, apdu.sw1, apdu.sw2));
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
 		ex_data->fix_digestInfo = env->algorithm_flags;
 		ex_data->sec_ops        = SC_SEC_OPERATION_AUTHENTICATE;
 		return SC_SUCCESS;
@@ -1153,7 +1143,7 @@ static int starcos_compute_signature(sc_card_t *card,
 	starcos_ex_data *ex_data = (starcos_ex_data *)card->drv_data;
 
 	if (datalen > SC_MAX_APDU_BUFFER_SIZE)
-		SC_FUNC_RETURN(card->ctx, 4, SC_ERROR_INVALID_ARGUMENTS);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
 
 	if (ex_data->sec_ops == SC_SEC_OPERATION_SIGN) {
 		/* compute signature with the COMPUTE SIGNATURE command */
@@ -1169,9 +1159,9 @@ static int starcos_compute_signature(sc_card_t *card,
 		apdu.lc = datalen;
 		apdu.datalen = datalen;
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 		if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
-			SC_FUNC_RETURN(card->ctx, 4, 
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, 
 				       sc_check_sw(card, apdu.sw1, apdu.sw2));
 
 		/* call COMPUTE SIGNATURE */
@@ -1183,13 +1173,12 @@ static int starcos_compute_signature(sc_card_t *card,
 
 		apdu.lc = 0;
 		apdu.datalen = 0;
-		apdu.sensitive = 1;
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 		if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 			size_t len = apdu.resplen > outlen ? outlen : apdu.resplen;
 			memcpy(out, apdu.resp, len);
-			SC_FUNC_RETURN(card->ctx, 4, len);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, len);
 		}
 	} else if (ex_data->sec_ops == SC_SEC_OPERATION_AUTHENTICATE) {
 		size_t tmp_len;
@@ -1217,21 +1206,21 @@ static int starcos_compute_signature(sc_card_t *card,
 		apdu.resplen = sizeof(rbuf);
 		apdu.le = 256;
 		r = sc_transmit_apdu(card, &apdu);
-		SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 		if (apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 			size_t len = apdu.resplen > outlen ? outlen : apdu.resplen;
 
 			memcpy(out, apdu.resp, len);
-			SC_FUNC_RETURN(card->ctx, 4, len);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, len);
 		}
 	} else
-		SC_FUNC_RETURN(card->ctx, 4, SC_ERROR_INVALID_ARGUMENTS);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
 
 	/* clear old state */
 	ex_data->sec_ops = 0;
 	ex_data->fix_digestInfo = 0;
 
-	SC_FUNC_RETURN(card->ctx, 4, sc_check_sw(card, apdu.sw1, apdu.sw2));
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, sc_check_sw(card, apdu.sw1, apdu.sw2));
 }
 
 static int starcos_check_sw(sc_card_t *card, unsigned int sw1, unsigned int sw2)
@@ -1239,14 +1228,14 @@ static int starcos_check_sw(sc_card_t *card, unsigned int sw1, unsigned int sw2)
 	const int err_count = sizeof(starcos_errors)/sizeof(starcos_errors[0]);
 	int i;
 
-	if (card->ctx->debug >= 3)
-		sc_debug(card->ctx, "sw1 = 0x%02x, sw2 = 0x%02x\n", sw1, sw2);
+	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+		"sw1 = 0x%02x, sw2 = 0x%02x\n", sw1, sw2);
   
 	if (sw1 == 0x90)
-		return SC_NO_ERROR;
+		return SC_SUCCESS;
 	if (sw1 == 0x63 && (sw2 & ~0x0fU) == 0xc0 )
 	{
-		sc_error(card->ctx, "Verification failed (remaining tries: %d)\n",
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Verification failed (remaining tries: %d)\n",
 		(sw2 & 0x0f));
 		return SC_ERROR_PIN_CODE_INCORRECT;
 	}
@@ -1255,7 +1244,7 @@ static int starcos_check_sw(sc_card_t *card, unsigned int sw1, unsigned int sw2)
 	for (i = 0; i < err_count; i++)
 		if (starcos_errors[i].SWs == ((sw1 << 8) | sw2))
 		{
-			sc_error(card->ctx, "%s\n", starcos_errors[i].errorstr);
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "%s\n", starcos_errors[i].errorstr);
 			return starcos_errors[i].errorno;
 		}
   
@@ -1285,12 +1274,12 @@ static int starcos_get_serialnr(sc_card_t *card, sc_serial_number_t *serial)
 	apdu.lc   = 0;
 	apdu.datalen = 0;
         r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00)
 		return SC_ERROR_INTERNAL;
 	/* cache serial number */
-	memcpy(card->serialnr.value, apdu.resp, apdu.resplen);
-	card->serialnr.len = apdu.resplen;
+	memcpy(card->serialnr.value, apdu.resp, MIN(apdu.resplen, SC_MAX_SERIALNR));
+	card->serialnr.len = MIN(apdu.resplen, SC_MAX_SERIALNR);
 	/* copy and return serial number */
 	memcpy(serial, &card->serialnr, sizeof(*serial));
 	return SC_SUCCESS;
@@ -1340,10 +1329,8 @@ static int starcos_logout(sc_card_t *card)
 	apdu.datalen = 2;
 	apdu.resplen = 0;
 	
-	sc_ctx_suppress_errors_on(card->ctx);
 	r = sc_transmit_apdu(card, &apdu);
-	sc_ctx_suppress_errors_off(card->ctx);
-	SC_TEST_RET(card->ctx, r, "APDU re-transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU re-transmit failed");
 
 	if (apdu.sw1 == 0x69 && apdu.sw2 == 0x85)
 		/* the only possible reason for this error here is, afaik,
