@@ -1179,7 +1179,7 @@ static int westcos_sign_decipher(int mode, sc_card_t *card,
 
 	/* pkcs11 reset openssl functions */
 	rsa->meth = RSA_PKCS1_SSLeay();
-	if (RSA_size(rsa) > outlen) {
+	if ((size_t)RSA_size(rsa) > outlen) {
 		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "Buffer too small\n");
 		r = SC_ERROR_OUT_OF_MEMORY;
 		goto out;
