@@ -36,7 +36,6 @@ extern "C" {
 #endif
 
 #include "common/simclist.h"
-#include "common/libscdl.h"
 #include "libopensc/opensc.h"
 #include "libopensc/log.h"
 #include "libopensc/cards.h"
@@ -124,9 +123,6 @@ int _sc_card_add_rsa_alg(struct sc_card *card, unsigned int key_length,
 int _sc_card_add_ec_alg(struct sc_card *card, unsigned int key_length,
 			 unsigned long flags, unsigned long ext_flags);
 
-int sc_asn1_read_tag(const u8 ** buf, size_t buflen, unsigned int *cla_out,
-		     unsigned int *tag_out, size_t *taglen);
-
 /********************************************************************/
 /*                 pkcs1 padding/encoding functions                 */
 /********************************************************************/
