@@ -59,13 +59,13 @@ static int generate_cache_filename(struct sc_pkcs15_card *p15card,
 	for (i = 0; i < pathlen; i++)
 		sprintf(pathname + 2*i, "%02X", pathptr[i]);
 	if (p15card->tokeninfo->serial_number != NULL) {
-		if (p15card->tokeninfo->last_update != NULL)
-			r = snprintf(buf, bufsize, "%s/%s_%s_%s", dir,
-			     p15card->tokeninfo->serial_number, p15card->tokeninfo->last_update,
-			     pathname);
+		char *last_update = sc_pkcs15_get_lastupdate(p15card);
+		if (last_update != NULL)
+			r = snprintf(buf, bufsize, "%s/%s_%s_%s", dir, p15card->tokeninfo->serial_number,
+					last_update, pathname);
 		else
 			r = snprintf(buf, bufsize, "%s/%s_DATE_%s", dir,
-			     p15card->tokeninfo->serial_number, pathname);
+					p15card->tokeninfo->serial_number, pathname);
 		if (r < 0)
 			return SC_ERROR_BUFFER_TOO_SMALL;
 	} else
