@@ -430,20 +430,21 @@ static int sc_pkcs15emu_gemsafeGPK_init(sc_pkcs15_card_t *p15card)
 	card->ops->pin_cmd = my_pin_cmd;
 
 	for (i = 0; pins[i].label; i++) {
-		struct sc_pkcs15_pin_info pin_info;
+		struct sc_pkcs15_auth_info pin_info;
 		struct sc_pkcs15_object   pin_obj;
 
 		memset(&pin_info, 0, sizeof(pin_info));
 		memset(&pin_obj,  0, sizeof(pin_obj));
 
 		sc_pkcs15_format_id(pins[i].id, &pin_info.auth_id);
-		pin_info.reference     = pins[i].ref;
-		pin_info.flags         = pins[i].flags;
-		pin_info.type          = pins[i].type;
-		pin_info.min_length    = pins[i].minlen;
-		pin_info.stored_length = pins[i].storedlen;
-		pin_info.max_length    = pins[i].maxlen;
-		pin_info.pad_char      = pins[i].pad_char;
+		pin_info.auth_type = SC_PKCS15_PIN_AUTH_TYPE_PIN;
+		pin_info.attrs.pin.reference     = pins[i].ref;
+		pin_info.attrs.pin.flags         = pins[i].flags;
+		pin_info.attrs.pin.type          = pins[i].type;
+		pin_info.attrs.pin.min_length    = pins[i].minlen;
+		pin_info.attrs.pin.stored_length = pins[i].storedlen;
+		pin_info.attrs.pin.max_length    = pins[i].maxlen;
+		pin_info.attrs.pin.pad_char      = pins[i].pad_char;
 		sc_format_path(pins[i].path, &pin_info.path);
 		pin_info.path.value[2] = dfpath >> 8;
 		pin_info.path.value[3] = dfpath & 0xff;
