@@ -26,6 +26,7 @@
 #include <stdio.h>
 #include <assert.h>
 
+#include "common/libscdl.h"
 #include "internal.h"
 #include "asn1.h"
 #include "pkcs15.h"
@@ -64,6 +65,8 @@ extern int sc_pkcs15emu_oberthur_init_ex(sc_pkcs15_card_t *,
 					sc_pkcs15emu_opt_t *);
 extern int sc_pkcs15emu_itacns_init_ex(sc_pkcs15_card_t *,
 					sc_pkcs15emu_opt_t *);
+extern int sc_pkcs15emu_sc_hsm_init_ex(sc_pkcs15_card_t *,
+					sc_pkcs15emu_opt_t *);
 
 static struct {
 	const char *		name;
@@ -86,6 +89,7 @@ static struct {
 	{ "entersafe",  sc_pkcs15emu_entersafe_init_ex  },
 	{ "pteid",	sc_pkcs15emu_pteid_init_ex	},
 	{ "oberthur",   sc_pkcs15emu_oberthur_init_ex	},
+	{ "sc-hsm",   sc_pkcs15emu_sc_hsm_init_ex	},
 	{ NULL, NULL }
 };
 
@@ -108,6 +112,7 @@ int sc_pkcs15_is_emulation_only(sc_card_t *card)
 		case SC_CARD_TYPE_GEMSAFEV1_PTEID:
 		case SC_CARD_TYPE_OPENPGP_V1:
 		case SC_CARD_TYPE_OPENPGP_V2:
+		case SC_CARD_TYPE_SC_HSM:
 			return 1;
 		default:
 			return 0;
