@@ -34,9 +34,9 @@ extern "C" {
 #define SC_PKCS15_MAX_LABEL_SIZE	255
 #define SC_PKCS15_MAX_ID_SIZE		255
 
-/* When changing this value, change also initialisation of the 
+/* When changing this value, change also initialisation of the
  * static ASN1 variables, that use this macro,
- * like for example, 'c_asn1_access_control_rules' 
+ * like for example, 'c_asn1_access_control_rules'
  * in src/libopensc/asn1.c */
 #define SC_PKCS15_MAX_ACCESS_RULES      8
 
@@ -63,6 +63,27 @@ typedef struct sc_pkcs15_id sc_pkcs15_id_t;
 #define SC_PKCS15_PIN_FLAG_CONFIDENTIALITY_PROTECTED	0x0400
 #define SC_PKCS15_PIN_FLAG_EXCHANGE_REF_DATA		0x0800
 
+#define SC_PKCS15_PIN_TYPE_FLAGS_MASK					\
+	( SC_PKCS15_PIN_FLAG_LOCAL | SC_PKCS15_PIN_FLAG_INITIALIZED 	\
+	| SC_PKCS15_PIN_FLAG_UNBLOCKING_PIN | SC_PKCS15_PIN_FLAG_SO_PIN )
+
+#define SC_PKCS15_PIN_TYPE_FLAGS_SOPIN					\
+	( SC_PKCS15_PIN_FLAG_SO_PIN | SC_PKCS15_PIN_FLAG_INITIALIZED )
+
+#define SC_PKCS15_PIN_TYPE_FLAGS_PIN_GLOBAL				\
+	( SC_PKCS15_PIN_FLAG_INITIALIZED )
+
+#define SC_PKCS15_PIN_TYPE_FLAGS_PIN_LOCAL				\
+	( SC_PKCS15_PIN_FLAG_INITIALIZED | SC_PKCS15_PIN_FLAG_LOCAL)
+
+#define SC_PKCS15_PIN_TYPE_FLAGS_PUK_GLOBAL				\
+	( SC_PKCS15_PIN_FLAG_UNBLOCKING_PIN 				\
+	| SC_PKCS15_PIN_FLAG_INITIALIZED )
+
+#define SC_PKCS15_PIN_TYPE_FLAGS_PUK_LOCAL				\
+	( SC_PKCS15_PIN_FLAG_UNBLOCKING_PIN 				\
+	| SC_PKCS15_PIN_FLAG_INITIALIZED | SC_PKCS15_PIN_FLAG_LOCAL)
+
 #define SC_PKCS15_PIN_TYPE_BCD				0
 #define SC_PKCS15_PIN_TYPE_ASCII_NUMERIC		1
 #define SC_PKCS15_PIN_TYPE_UTF8				2
@@ -73,7 +94,6 @@ typedef struct sc_pkcs15_id sc_pkcs15_id_t;
 #define SC_PKCS15_PIN_AUTH_TYPE_BIOMETRIC		1
 #define SC_PKCS15_PIN_AUTH_TYPE_AUTH_KEY		2
 #define SC_PKCS15_PIN_AUTH_TYPE_SM_KEY			3
-
 /* PinAttributes as they defined in PKCS#15 v1.1 for PIN authentication object */
 struct sc_pkcs15_pin_attributes {
 	unsigned int  flags, type;
@@ -173,7 +193,7 @@ struct sc_pkcs15_prkey_dsa {
 	sc_pkcs15_bignum_t priv;
 };
 
-/* 
+/*
  * The ecParameters can be presented as
  * - named curve;
  * - OID of named curve;
@@ -200,6 +220,7 @@ struct sc_pkcs15_pubkey_ec {
 struct sc_pkcs15_prkey_ec {
 	struct sc_pkcs15_ec_parameters params;
 	sc_pkcs15_bignum_t	privateD; /* note this is bignum */
+	sc_pkcs15_der_t		ecpointQ; /* note this is der */
 };
 
 struct sc_pkcs15_pubkey_gostr3410 {
@@ -267,8 +288,9 @@ struct sc_pkcs15_cert {
 	size_t crl_len;
 
 	struct sc_pkcs15_pubkey * key;
-	u8 *data;	/* DER encoded raw cert */
-	size_t data_len;
+
+	/* DER encoded raw cert */
+	struct sc_pkcs15_der data;
 };
 typedef struct sc_pkcs15_cert sc_pkcs15_cert_t;
 
@@ -312,6 +334,18 @@ typedef struct sc_pkcs15_data_info sc_pkcs15_data_info_t;
 #define SC_PKCS15_PRKEY_USAGE_DERIVE		0x100
 #define SC_PKCS15_PRKEY_USAGE_NONREPUDIATION	0x200
 
+/* keyUsageFlags  are the same for all key types */
+#define SC_PKCS15_KEY_USAGE_ENCRYPT		0x01
+#define SC_PKCS15_KEY_USAGE_DECRYPT		0x02
+#define SC_PKCS15_KEY_USAGE_SIGN		0x04
+#define SC_PKCS15_KEY_USAGE_SIGNRECOVER		0x08
+#define SC_PKCS15_KEY_USAGE_WRAP		0x10
+#define SC_PKCS15_KEY_USAGE_UNWRAP		0x20
+#define SC_PKCS15_KEY_USAGE_VERIFY		0x40
+#define SC_PKCS15_KEY_USAGE_VERIFYRECOVER	0x80
+#define SC_PKCS15_KEY_USAGE_DERIVE		0x100
+#define SC_PKCS15_KEY_USAGE_NONREPUDIATION	0x200
+
 #define SC_PKCS15_PRKEY_ACCESS_SENSITIVE	0x01
 #define SC_PKCS15_PRKEY_ACCESS_EXTRACTABLE	0x02
 #define SC_PKCS15_PRKEY_ACCESS_ALWAYSSENSITIVE	0x04
@@ -392,6 +426,21 @@ struct sc_pkcs15_pubkey_info {
 };
 typedef struct sc_pkcs15_pubkey_info sc_pkcs15_pubkey_info_t;
 
+struct sc_pkcs15_skey_info {
+	struct sc_pkcs15_id id;
+	unsigned int usage, access_flags;
+	int native, key_reference;
+	size_t value_len;
+	unsigned long key_type;
+	int algo_refs[SC_MAX_SUPPORTED_ALGORITHMS];
+	struct sc_path path; /* if on card */
+	struct sc_pkcs15_der data;
+};
+typedef struct sc_pkcs15_skey_info sc_pkcs15_skey_info_t;
+
+#define sc_pkcs15_skey sc_pkcs15_data
+#define sc_pkcs15_skey_t sc_pkcs15_data_t
+
 #define SC_PKCS15_TYPE_CLASS_MASK		0xF00
 
 #define SC_PKCS15_TYPE_PRKEY			0x100
@@ -406,17 +455,27 @@ typedef struct sc_pkcs15_pubkey_info sc_pkcs15_pubkey_info_t;
 #define SC_PKCS15_TYPE_PUBKEY_GOSTR3410		0x203
 #define SC_PKCS15_TYPE_PUBKEY_EC		0x204
 
+#define SC_PKCS15_TYPE_SKEY			0x300
+#define SC_PKCS15_TYPE_SKEY_GENERIC		0x301
+#define SC_PKCS15_TYPE_SKEY_DES			0x302
+#define SC_PKCS15_TYPE_SKEY_2DES		0x303
+#define SC_PKCS15_TYPE_SKEY_3DES		0x304
+
 #define SC_PKCS15_TYPE_CERT			0x400
 #define SC_PKCS15_TYPE_CERT_X509		0x401
 #define SC_PKCS15_TYPE_CERT_SPKI		0x402
 
 #define SC_PKCS15_TYPE_DATA_OBJECT		0x500
+
 #define SC_PKCS15_TYPE_AUTH			0x600
 #define SC_PKCS15_TYPE_AUTH_PIN			0x601
+#define SC_PKCS15_TYPE_AUTH_BIO			0x602
+#define SC_PKCS15_TYPE_AUTH_AUTHKEY		0x603
 
 #define SC_PKCS15_TYPE_TO_CLASS(t)		(1 << ((t) >> 8))
 #define SC_PKCS15_SEARCH_CLASS_PRKEY		0x0002U
 #define SC_PKCS15_SEARCH_CLASS_PUBKEY		0x0004U
+#define SC_PKCS15_SEARCH_CLASS_SKEY		0x0008U
 #define SC_PKCS15_SEARCH_CLASS_CERT		0x0010U
 #define SC_PKCS15_SEARCH_CLASS_DATA		0x0020U
 #define SC_PKCS15_SEARCH_CLASS_AUTH		0x0040U
@@ -438,11 +497,13 @@ struct sc_pkcs15_object {
 	/* emulated object pointer */
 	void *emulated;
 
-
 	struct sc_pkcs15_df *df; /* can be NULL, if object is 'floating' */
 	struct sc_pkcs15_object *next, *prev; /* used only internally */
-	
+
 	struct sc_pkcs15_der content;
+
+	/* Used by minidriver and its on-card support */
+	char *guid;
 };
 typedef struct sc_pkcs15_object sc_pkcs15_object_t;
 
@@ -486,13 +547,27 @@ typedef struct sc_pkcs15_sec_env_info {
 	struct sc_aid aid;
 } sc_pkcs15_sec_env_info_t;
 
+typedef struct sc_pkcs15_last_update {
+	char *gtime;
+	struct sc_path path;
+
+} sc_pkcs15_last_update_t;
+
+typedef struct sc_pkcs15_profile_indication {
+	struct sc_object_id oid;
+	char *name;
+} sc_pkcs15_profile_indication_t;
+
 typedef struct sc_pkcs15_tokeninfo {
 	unsigned int version;
 	unsigned int flags;
 	char *label;
 	char *serial_number;
-	char *manufacturer_id;	
-	char *last_update;
+	char *manufacturer_id;
+
+	struct sc_pkcs15_last_update last_update;
+	struct sc_pkcs15_profile_indication profile_indication;
+
 	char *preferred_language;
 	sc_pkcs15_sec_env_info_t **seInfo;
 	size_t num_seInfo;
@@ -503,7 +578,7 @@ typedef struct sc_pkcs15_tokeninfo {
 struct sc_pkcs15_operations   {
 	int (*parse_df)(struct sc_pkcs15_card *, struct sc_pkcs15_df *);
 	void (*clear)(struct sc_pkcs15_card *);
-	int (*get_guid)(struct sc_pkcs15_card *, const struct sc_pkcs15_object *, 
+	int (*get_guid)(struct sc_pkcs15_card *, const struct sc_pkcs15_object *,
 			char *, size_t);
 };
 
@@ -526,9 +601,9 @@ typedef struct sc_pkcs15_card {
 		int use_file_cache;
 		int use_pin_cache;
 		int pin_cache_counter;
+		int pin_cache_ignore_user_consent;
 	} opts;
 
-
 	unsigned int magic;
 
 	void *dll_handle;		/* shared lib for emulated cards */
@@ -575,6 +650,11 @@ int sc_pkcs15_decipher(struct sc_pkcs15_card *p15card,
 		       unsigned long flags,
 		       const u8 *in, size_t inlen, u8 *out, size_t outlen);
 
+int sc_pkcs15_derive(struct sc_pkcs15_card *p15card,
+		       const struct sc_pkcs15_object *prkey_obj,
+		       unsigned long flags,
+		       const u8 *in, size_t inlen, u8 *out, unsigned long *poutlen);
+
 int sc_pkcs15_compute_signature(struct sc_pkcs15_card *p15card,
 				const struct sc_pkcs15_object *prkey_obj,
 				unsigned long alg_flags, const u8 *in,
@@ -607,11 +687,11 @@ int sc_pkcs15_encode_pubkey(struct sc_context *,
 			struct sc_pkcs15_pubkey *, u8 **, size_t *);
 void sc_pkcs15_erase_pubkey(struct sc_pkcs15_pubkey *);
 void sc_pkcs15_free_pubkey(struct sc_pkcs15_pubkey *);
-int sc_pkcs15_pubkey_from_prvkey(struct sc_context *, struct sc_pkcs15_prkey *, 
+int sc_pkcs15_pubkey_from_prvkey(struct sc_context *, struct sc_pkcs15_prkey *,
 			struct sc_pkcs15_pubkey **);
-int sc_pkcs15_pubkey_from_cert(struct sc_context *, struct sc_pkcs15_der *, 
+int sc_pkcs15_pubkey_from_cert(struct sc_context *, struct sc_pkcs15_der *,
 			struct sc_pkcs15_pubkey **);
-int sc_pkcs15_pubkey_from_spki_filename(struct sc_context *, 
+int sc_pkcs15_pubkey_from_spki_filename(struct sc_context *,
 			char *, sc_pkcs15_pubkey_t ** );
 int sc_pkcs15_pubkey_from_spki(struct sc_context *,
 			sc_pkcs15_pubkey_t **, u8 *, size_t, int);
@@ -664,6 +744,9 @@ int sc_pkcs15_find_prkey_by_reference(sc_pkcs15_card_t *,
 int sc_pkcs15_find_pubkey_by_id(struct sc_pkcs15_card *card,
 			       const struct sc_pkcs15_id *id,
 			       struct sc_pkcs15_object **out);
+int sc_pkcs15_find_skey_by_id(struct sc_pkcs15_card *card,
+			       const struct sc_pkcs15_id *id,
+			       struct sc_pkcs15_object **out);
 
 int sc_pkcs15_verify_pin(struct sc_pkcs15_card *card,
 			 struct sc_pkcs15_object *pin_obj,
@@ -683,15 +766,18 @@ int sc_pkcs15_find_pin_by_reference(struct sc_pkcs15_card *card,
 				    const sc_path_t *path, int reference,
 				    struct sc_pkcs15_object **out);
 int sc_pkcs15_find_pin_by_type_and_reference(struct sc_pkcs15_card *card,
-				    const sc_path_t *path, unsigned auth_method, 
+				    const sc_path_t *path, unsigned auth_method,
 				    int reference,
 				    struct sc_pkcs15_object **out);
 int sc_pkcs15_find_so_pin(struct sc_pkcs15_card *card,
 			struct sc_pkcs15_object **out);
+int sc_pkcs15_find_pin_by_flags(struct sc_pkcs15_card *p15card,
+		unsigned flags, unsigned mask, int *index,
+		struct sc_pkcs15_object **out);
 
-void sc_pkcs15_pincache_add(struct sc_pkcs15_card *, struct sc_pkcs15_object *, 
+void sc_pkcs15_pincache_add(struct sc_pkcs15_card *, struct sc_pkcs15_object *,
 			const u8 *, size_t);
-int sc_pkcs15_pincache_revalidate(struct sc_pkcs15_card *p15card, 
+int sc_pkcs15_pincache_revalidate(struct sc_pkcs15_card *p15card,
 			const sc_pkcs15_object_t *obj);
 void sc_pkcs15_pincache_clear(struct sc_pkcs15_card *p15card);
 
@@ -746,6 +832,9 @@ int sc_pkcs15_decode_prkdf_entry(struct sc_pkcs15_card *p15card,
 int sc_pkcs15_decode_pukdf_entry(struct sc_pkcs15_card *p15card,
 				 struct sc_pkcs15_object *obj,
 				 const u8 **buf, size_t *bufsize);
+int sc_pkcs15_decode_skdf_entry(struct sc_pkcs15_card *p15card,
+				 struct sc_pkcs15_object *obj,
+				 const u8 **buf, size_t *bufsize);
 
 int sc_pkcs15_decode_enveloped_data(struct sc_context *ctx,
 				    struct sc_pkcs15_enveloped_data *result,
@@ -759,19 +848,19 @@ int sc_pkcs15_add_object(struct sc_pkcs15_card *p15card,
 void sc_pkcs15_remove_object(struct sc_pkcs15_card *p15card,
 			     struct sc_pkcs15_object *obj);
 int sc_pkcs15_add_df(struct sc_pkcs15_card *, unsigned int, const sc_path_t *);
-void sc_pkcs15_remove_df(struct sc_pkcs15_card *p15card,
-			 struct sc_pkcs15_df *df);
 
 int sc_pkcs15_add_unusedspace(struct sc_pkcs15_card *p15card,
 		     const sc_path_t *path, const sc_pkcs15_id_t *auth_id);
-void sc_pkcs15_remove_unusedspace(struct sc_pkcs15_card *p15card,
-			 sc_pkcs15_unusedspace_t *obj);
 int sc_pkcs15_parse_unusedspace(const u8 * buf, size_t buflen,
 			struct sc_pkcs15_card *card);
 int sc_pkcs15_encode_unusedspace(sc_context_t *ctx,
 			 struct sc_pkcs15_card *p15card,
 			 u8 **buf, size_t *buflen);
 
+/* Deduce private key attributes from cerresponding certificate */
+int sc_pkcs15_prkey_attrs_from_cert(struct sc_pkcs15_card *, struct sc_pkcs15_object *,
+		struct sc_pkcs15_object **);
+
 void sc_pkcs15_free_prkey_info(sc_pkcs15_prkey_info_t *key);
 void sc_pkcs15_free_pubkey_info(sc_pkcs15_pubkey_info_t *key);
 void sc_pkcs15_free_cert_info(sc_pkcs15_cert_info_t *cert);
@@ -800,11 +889,14 @@ void sc_pkcs15_format_id(const char *id_in, struct sc_pkcs15_id *id_out);
 int sc_pkcs15_hex_string_to_id(const char *in, struct sc_pkcs15_id *out);
 int sc_der_copy(sc_pkcs15_der_t *, const sc_pkcs15_der_t *);
 int sc_pkcs15_get_object_id(const struct sc_pkcs15_object *, struct sc_pkcs15_id *);
-int sc_pkcs15_get_guid(struct sc_pkcs15_card *, const struct sc_pkcs15_object *, 
+int sc_pkcs15_get_guid(struct sc_pkcs15_card *, const struct sc_pkcs15_object *, unsigned,
 		char *, size_t);
-int sc_encode_oid (struct sc_context *, struct sc_object_id *, 
+int sc_encode_oid (struct sc_context *, struct sc_object_id *,
 		unsigned char **, size_t *);
 
+/* Get application by type: 'protected', 'generic' */
+struct sc_app_info *sc_pkcs15_get_application_by_type(struct sc_card *, char *);
+
 /* Prepend 'parent' to 'child' in case 'child' is a relative path */
 int sc_pkcs15_make_absolute_path(const sc_path_t *parent, sc_path_t *child);
 
@@ -812,7 +904,7 @@ int sc_pkcs15_make_absolute_path(const sc_path_t *parent, sc_path_t *child);
 void sc_pkcs15_free_object_content(struct sc_pkcs15_object *);
 
 /* Allocate and set object content */
-int sc_pkcs15_allocate_object_content(struct sc_pkcs15_object *,
+int sc_pkcs15_allocate_object_content(struct sc_context *, struct sc_pkcs15_object *,
 		const unsigned char *, size_t);
 
 struct sc_supported_algo_info *sc_pkcs15_get_supported_algo(struct sc_pkcs15_card *,
@@ -822,6 +914,14 @@ int sc_pkcs15_add_supported_algo_ref(struct sc_pkcs15_object *,
 
 int sc_pkcs15_fix_ec_parameters(struct sc_context *, struct sc_pkcs15_ec_parameters *);
 
+/* Convert the OpenSSL key data type into the OpenSC key */
+int sc_pkcs15_convert_bignum(sc_pkcs15_bignum_t *dst, const void *bignum);
+int sc_pkcs15_convert_prkey(struct sc_pkcs15_prkey *key, void *evp_key);
+int sc_pkcs15_convert_pubkey(struct sc_pkcs15_pubkey *key, void *evp_key);
+
+/* Get 'LastUpdate' string */
+char *sc_pkcs15_get_lastupdate(struct sc_pkcs15_card *p15card);
+
 /* New object search API.
  * More complex, but also more powerful.
  */
