@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "common/libscdl.h"
 #include "internal.h"
 #include "ctbcs.h"
 
@@ -562,6 +563,7 @@ struct sc_reader_driver * sc_get_ctapi_driver(void)
 	ctapi_ops.connect = ctapi_connect;
 	ctapi_ops.disconnect = ctapi_disconnect;
 	ctapi_ops.perform_verify = ctbcs_pin_cmd;
+	ctapi_ops.perform_pace = NULL;
 	ctapi_ops.use_reader = NULL;
 	
 	return &ctapi_drv;
