@@ -150,7 +150,7 @@ static int sc_unlock_mutex(void *m)
 		return SC_SUCCESS;
 	else
 		return SC_ERROR_INTERNAL;
-	
+
 }
 
 static int sc_destroy_mutex(void *m)
@@ -208,7 +208,7 @@ CK_RV C_Initialize(CK_VOID_PTR pInitArgs)
 #endif
 
 	if (context != NULL) {
-		sc_debug(context, SC_LOG_DEBUG_NORMAL, "C_Initialize(): Cryptoki already initialized\n");
+		sc_log(context, "C_Initialize(): Cryptoki already initialized\n");
 		return CKR_CRYPTOKI_ALREADY_INITIALIZED;
 	}
 
@@ -221,7 +221,7 @@ CK_RV C_Initialize(CK_VOID_PTR pInitArgs)
 	ctx_opts.ver        = 0;
 	ctx_opts.app_name   = "opensc-pkcs11";
 	ctx_opts.thread_ctx = &sc_thread_ctx;
-	
+
 	rc = sc_context_create(&context, &ctx_opts);
 	if (rc != SC_SUCCESS) {
 		rv = CKR_GENERAL_ERROR;
@@ -234,11 +234,11 @@ CK_RV C_Initialize(CK_VOID_PTR pInitArgs)
 	/* List of sessions */
 	list_init(&sessions);
 	list_attributes_seeker(&sessions, session_list_seeker);
-	
+
 	/* List of slots */
 	list_init(&virtual_slots);
 	list_attributes_seeker(&virtual_slots, slot_list_seeker);
-	
+
 	/* Create a slot for a future "PnP" stuff. */
 	if (sc_pkcs11_conf.plug_and_play) {
 		create_slot(NULL);
@@ -254,9 +254,9 @@ CK_RV C_Initialize(CK_VOID_PTR pInitArgs)
 		slot->events = 0; /* Initially there are no events */
 	}
 
-out:	
+out:
 	if (context != NULL)
-		sc_debug(context, SC_LOG_DEBUG_NORMAL, "C_Initialize() = %s", lookup_enum ( RV_T, rv ));
+		sc_log(context, "C_Initialize() = %s", lookup_enum ( RV_T, rv ));
 
 	if (rv != CKR_OK) {
 		if (context != NULL) {
@@ -287,8 +287,8 @@ CK_RV C_Finalize(CK_VOID_PTR pReserved)
 	if (rv != CKR_OK)
 		return rv;
 
-	sc_debug(context, SC_LOG_DEBUG_NORMAL, "C_Finalize()");
-	
+	sc_log(context, "C_Finalize()");
+
 	/* cancel pending calls */
 	in_finalize = 1;
 	sc_cancel(context);
@@ -326,7 +326,7 @@ CK_RV C_GetInfo(CK_INFO_PTR pInfo)
 	if (rv != CKR_OK)
 		return rv;
 
-	sc_debug(context, SC_LOG_DEBUG_NORMAL, "C_GetInfo()");
+	sc_log(context, "C_GetInfo()");
 
 	memset(pInfo, 0, sizeof(CK_INFO));
 	pInfo->cryptokiVersion.major = 2;
@@ -342,7 +342,7 @@ CK_RV C_GetInfo(CK_INFO_PTR pInfo)
 
 	sc_pkcs11_unlock();
 	return rv;
-}	
+}
 
 CK_RV C_GetFunctionList(CK_FUNCTION_LIST_PTR_PTR ppFunctionList)
 {
@@ -371,7 +371,7 @@ CK_RV C_GetSlotList(CK_BBOOL       tokenPresent,  /* only slots with token prese
 		return rv;
 	}
 
-	sc_debug(context, SC_LOG_DEBUG_NORMAL, "C_GetSlotList(token=%d, %s)", tokenPresent,
+	sc_log(context, "C_GetSlotList(token=%d, %s)", tokenPresent,
 		 (pSlotList==NULL_PTR && sc_pkcs11_conf.plug_and_play)? "plug-n-play":"refresh");
 
 	/* Slot list can only change in v2.20 */
@@ -379,7 +379,7 @@ CK_RV C_GetSlotList(CK_BBOOL       tokenPresent,  /* only slots with token prese
 		/* Trick NSS into updating the slot list by changing the hotplug slot ID */
 		sc_pkcs11_slot_t *hotplug_slot = list_get_at(&virtual_slots, 0);
 		hotplug_slot->id--;
-		sc_ctx_detect_readers(context); 
+		sc_ctx_detect_readers(context);
 	}
 
 	card_detect_all();
@@ -400,22 +400,22 @@ CK_RV C_GetSlotList(CK_BBOOL       tokenPresent,  /* only slots with token prese
 		 * - any slot with token;
 		 * - without token(s), one empty slot per reader;
 		 */
-	        if ((!tokenPresent && !slot->reader) 
-				|| (!tokenPresent && slot->reader != prev_reader) 
+	        if ((!tokenPresent && !slot->reader)
+				|| (!tokenPresent && slot->reader != prev_reader)
 				|| (slot->slot_info.flags & CKF_TOKEN_PRESENT))
 			found[numMatches++] = slot->id;
 		prev_reader = slot->reader;
 	}
 
 	if (pSlotList == NULL_PTR) {
-		sc_debug(context, SC_LOG_DEBUG_NORMAL, "was only a size inquiry (%d)\n", numMatches);
+		sc_log(context, "was only a size inquiry (%d)\n", numMatches);
 		*pulCount = numMatches;
 		rv = CKR_OK;
 		goto out;
 	}
 
 	if (*pulCount < numMatches) {
-		sc_debug(context, SC_LOG_DEBUG_NORMAL, "buffer was too small (needed %d)\n", numMatches);
+		sc_log(context, "buffer was too small (needed %d)\n", numMatches);
 		*pulCount = numMatches;
 		rv = CKR_BUFFER_TOO_SMALL;
 		goto out;
@@ -425,7 +425,7 @@ CK_RV C_GetSlotList(CK_BBOOL       tokenPresent,  /* only slots with token prese
 	*pulCount = numMatches;
 	rv = CKR_OK;
 
-	sc_debug(context, SC_LOG_DEBUG_NORMAL, "returned %d slots\n", numMatches);
+	sc_log(context, "returned %d slots\n", numMatches);
 
 out:
 	if (found != NULL) {
@@ -476,7 +476,7 @@ CK_RV C_GetSlotInfo(CK_SLOT_ID slotID, CK_SLOT_INFO_PTR pInfo)
 	if (rv != CKR_OK)
 		return rv;
 
-	sc_debug(context, SC_LOG_DEBUG_NORMAL, "C_GetSlotInfo(0x%lx)", slotID);
+	sc_log(context, "C_GetSlotInfo(0x%lx)", slotID);
 
 	rv = slot_get_slot(slotID, &slot);
 	if (rv == CKR_OK){
@@ -498,7 +498,7 @@ CK_RV C_GetSlotInfo(CK_SLOT_ID slotID, CK_SLOT_INFO_PTR pInfo)
 	if (rv == CKR_OK)
 		memcpy(pInfo, &slot->slot_info, sizeof(CK_SLOT_INFO));
 
-	sc_debug(context, SC_LOG_DEBUG_NORMAL, "C_GetSlotInfo(0x%lx) = %s", slotID, lookup_enum ( RV_T, rv ));
+	sc_log(context, "C_GetSlotInfo(0x%lx) = %s", slotID, lookup_enum ( RV_T, rv ));
 	sc_pkcs11_unlock();
 	return rv;
 }
@@ -564,7 +564,7 @@ CK_RV C_InitToken(CK_SLOT_ID slotID,
 	rv = slot_get_token(slotID, &slot);
 	if (rv != CKR_OK)
 		goto out;
-	
+
 	/* Make sure there's no open session for this token */
 	for (i=0; i<list_size(&sessions); i++) {
 		session = (struct sc_pkcs11_session*)list_get_at(&sessions, i);
@@ -600,11 +600,11 @@ CK_RV C_WaitForSlotEvent(CK_FLAGS flags,   /* blocking/nonblocking flag */
 	CK_SLOT_ID slot_id;
 	CK_RV rv;
 	int r;
-	
+
 	if (pReserved != NULL_PTR)
 		return  CKR_ARGUMENTS_BAD;
 
-	sc_debug(context, SC_LOG_DEBUG_NORMAL, "C_WaitForSlotEvent(block=%d)", !(flags & CKF_DONT_BLOCK));
+	sc_log(context, "C_WaitForSlotEvent(block=%d)", !(flags & CKF_DONT_BLOCK));
 	/* Not all pcsc-lite versions implement consistently used functions as they are */
 	/* FIXME: add proper checking into build to check correct pcsc-lite version for SCardStatusChange/SCardCancel */
 	if (!(flags & CKF_DONT_BLOCK))
@@ -625,7 +625,7 @@ CK_RV C_WaitForSlotEvent(CK_FLAGS flags,   /* blocking/nonblocking flag */
 		goto out;
 
 again:
-	sc_debug(context, SC_LOG_DEBUG_NORMAL, "C_WaitForSlotEvent() reader_states:%p", reader_states);
+	sc_log(context, "C_WaitForSlotEvent() reader_states:%p", reader_states);
 	sc_pkcs11_unlock();
 	r = sc_wait_for_event(context, mask, &found, &events, -1, &reader_states);
 	if (sc_pkcs11_conf.plug_and_play && events & SC_EVENT_READER_ATTACHED) {
@@ -633,7 +633,7 @@ again:
 		   Change the first hotplug slot id on every call to make this happen. */
 		sc_pkcs11_slot_t *hotplug_slot = list_get_at(&virtual_slots, 0);
 		*pSlot= hotplug_slot->id -1;
-	
+
 		rv = sc_pkcs11_lock();
 		if (rv != CKR_OK)
 			return rv;
@@ -648,7 +648,7 @@ again:
 		return rv;
 
 	if (r != SC_SUCCESS) {
-		sc_debug(context, SC_LOG_DEBUG_NORMAL, "sc_wait_for_event() returned %d\n",  r);
+		sc_log(context, "sc_wait_for_event() returned %d\n",  r);
 		rv = sc_to_cryptoki_error(r, "C_WaitForSlotEvent");
 		goto out;
 	}
@@ -659,17 +659,17 @@ again:
 	if (rv != CKR_OK)
 		goto again;
 
-out:	
+out:
 	if (pSlot)
 		*pSlot = slot_id;
 
 	/* Free allocated readers states holder */
 	if (reader_states)   {
-		sc_debug(context, SC_LOG_DEBUG_NORMAL, "free reader states");
+		sc_log(context, "free reader states");
 		sc_wait_for_event(context, 0, NULL, NULL, -1, &reader_states);
 	}
 
-	sc_debug(context, SC_LOG_DEBUG_NORMAL, "C_WaitForSlotEvent() = %s, event in 0x%lx", lookup_enum (RV_T, rv), *pSlot);
+	sc_log(context, "C_WaitForSlotEvent() = %s, event in 0x%lx", lookup_enum (RV_T, rv), *pSlot);
 	sc_pkcs11_unlock();
 	return rv;
 }
@@ -740,7 +740,7 @@ CK_RV sc_pkcs11_lock(void)
 	if (global_locking)  {
 		while (global_locking->LockMutex(global_lock) != CKR_OK)
 			;
-	} 
+	}
 
 	return CKR_OK;
 }
@@ -753,7 +753,7 @@ __sc_pkcs11_unlock(void *lock)
 	if (global_locking) {
 		while (global_locking->UnlockMutex(lock) != CKR_OK)
 			;
-	} 
+	}
 }
 
 void sc_pkcs11_unlock(void)
