@@ -68,7 +68,7 @@ static int dump_unusedspace(void)
 	}
 	path.count = -1;
 
-	r = sc_pkcs15_read_file(p15card, &path, &buf, &buf_len, NULL);
+	r = sc_pkcs15_read_file(p15card, &path, &buf, &buf_len);
 	if (r < 0) {
 		if (r == SC_ERROR_FILE_NOT_FOUND) {
 			printf("\nNo EF(UnusedSpace) file\n");
