@@ -63,7 +63,9 @@
 #include "libopensc/pkcs15.h"
 #include "libopensc/log.h"
 #include "libopensc/cards.h"
+#include "libopensc/asn1.h"
 #include "pkcs15init/pkcs15-init.h"
+#include "pkcs15init/profile.h"
 #include "util.h"
 
 #undef GET_KEY_ECHO_OFF
@@ -78,6 +80,7 @@ typedef int	(*pkcs15_encoder)(sc_context_t *,
 static int	open_reader_and_card(char *);
 static int	do_assert_pristine(sc_card_t *);
 static int	do_erase(sc_card_t *, struct sc_profile *);
+static int	do_erase_application(sc_card_t *, struct sc_profile *);
 static int	do_delete_objects(struct sc_profile *, unsigned int myopt_delete_flags);
 static int	do_change_attributes(struct sc_profile *, unsigned int myopt_type);
 static int	do_init_app(struct sc_profile *);
@@ -87,8 +90,6 @@ static int	do_store_private_key(struct sc_profile *);
 static int	do_store_public_key(struct sc_profile *, EVP_PKEY *);
 static int	do_store_certificate(struct sc_profile *);
 static int	do_update_certificate(struct sc_profile *);
-static int	do_convert_private_key(struct sc_pkcs15_prkey *, EVP_PKEY *);
-static int	do_convert_public_key(struct sc_pkcs15_pubkey *, EVP_PKEY *);
 static int	do_convert_cert(sc_pkcs15_der_t *, X509 *);
 static int	is_cacert_already_present(struct sc_pkcs15init_certargs *);
 static int	do_finalize_card(sc_card_t *, struct sc_profile *);
@@ -107,10 +108,10 @@ static int	get_key_callback(struct sc_profile *,
 			int method, int reference,
 			const u8 *, size_t, u8 *, size_t *);
 
-static int	do_read_private_key(const char *, const char *,
-				EVP_PKEY **, X509 **, unsigned int);
+static int	do_read_private_key(const char *, const char *, EVP_PKEY **, X509 **, unsigned int);
 static int	do_read_public_key(const char *, const char *, EVP_PKEY **);
 static int	do_read_certificate(const char *, const char *, X509 **);
+static char *	cert_common_name(X509 *x509);
 static void	parse_commandline(int argc, char **argv);
 static void	read_options_file(const char *);
 static void	ossl_print_errors(void);
@@ -134,6 +135,9 @@ enum {
 	OPT_VERIFY_PIN,
 	OPT_SANITY_CHECK,
 	OPT_BIND_TO_AID,
+	OPT_UPDATE_LAST_UPDATE,
+	OPT_ERASE_APPLICATION,
+	OPT_IGNORE_CA_CERTIFICATES,
 
 	OPT_PIN1     = 0x10000,	/* don't touch these values */
 	OPT_PUK1     = 0x10001,
@@ -157,6 +161,7 @@ const struct option	options[] = {
 	{ "delete-objects",	required_argument, NULL,	'D' },
 	{ "change-attributes",	required_argument, NULL,	'A' },
 	{ "sanity-check",	no_argument, NULL,		OPT_SANITY_CHECK},
+	{ "erase-application",	required_argument, NULL,	OPT_ERASE_APPLICATION},
 
 	{ "reader",		required_argument, NULL,	'r' },
 	{ "pin",		required_argument, NULL,	OPT_PIN1 },
@@ -181,7 +186,9 @@ const struct option	options[] = {
 	{ "passphrase",		required_argument, NULL,	OPT_PASSPHRASE },
 	{ "authority",		no_argument,	   NULL,	OPT_AUTHORITY },
 	{ "key-usage",		required_argument, NULL,	'u' },
-	{ "finalize",		no_argument,       NULL,   	'F' },
+	{ "finalize",		no_argument,       NULL,	'F' },
+	{ "update-last-update", no_argument,       NULL,        OPT_UPDATE_LAST_UPDATE},
+	{ "ignore-ca-certificates",no_argument,    NULL,	OPT_IGNORE_CA_CERTIFICATES},
 
 	{ "extractable",	no_argument, NULL,		OPT_EXTRACTABLE },
 	{ "insecure",		no_argument, NULL,		OPT_INSECURE },
@@ -214,6 +221,7 @@ static const char *		option_help[] = {
 	"Delete object(s) (use \"help\" for more information)",
 	"Change attribute(s) (use \"help\" for more information)",
 	"Card specific sanity check and possibly update procedure",
+	"Erase application with AID <arg>",
 
 	"Specify which reader to use",
 	"Specify PIN",
@@ -239,6 +247,8 @@ static const char *		option_help[] = {
 	"Mark certificate as a CA certificate",
 	"Specify X.509 key usage (use \"--key-usage help\" for more information)",
 	"Finish initialization phase of the smart card",
+	"Update 'lastUpdate' attribut of tokenInfo",
+	"When storing PKCS#12 ignore CA certificates",
 
 	"Private key stored as an extractable key",
 	"Insecure mode: do not require a PIN for private key",
@@ -261,6 +271,7 @@ enum {
 	ACTION_ASSERT_PRISTINE,
 	ACTION_ERASE,
 	ACTION_INIT,
+	ACTION_DELETE_OBJECTS,
 	ACTION_STORE_PIN,
 	ACTION_GENERATE_KEY,
 	ACTION_STORE_PRIVKEY,
@@ -269,9 +280,10 @@ enum {
 	ACTION_UPDATE_CERT,
 	ACTION_STORE_DATA,
 	ACTION_FINALIZE_CARD,
-	ACTION_DELETE_OBJECTS,
 	ACTION_CHANGE_ATTRIBUTES,
 	ACTION_SANITY_CHECK,
+	ACTION_UPDATE_LAST_UPDATE,
+	ACTION_ERASE_APPLICATION,
 
 	ACTION_MAX
 };
@@ -348,6 +360,7 @@ static unsigned int		opt_type = 0;
 static int			ignore_cmdline_pins = 0;
 static struct secret		opt_secrets[MAX_SECRETS];
 static unsigned int		opt_secret_count;
+static int			opt_ignore_ca_certs = 0;
 static int			verbose = 0;
 
 static struct sc_pkcs15init_callbacks callbacks = {
@@ -420,14 +433,14 @@ main(int argc, char **argv)
 	parse_commandline(argc, argv);
 
 	if (optind != argc)
-		util_print_usage_and_die(app_name, options, option_help);
+		util_print_usage_and_die(app_name, options, option_help, NULL);
 	if (opt_actions == 0) {
 		fprintf(stderr, "No action specified.\n");
-		util_print_usage_and_die(app_name, options, option_help);
+		util_print_usage_and_die(app_name, options, option_help, NULL);
 	}
 	if (!opt_profile) {
 		fprintf(stderr, "No profile specified.\n");
-		util_print_usage_and_die(app_name, options, option_help);
+		util_print_usage_and_die(app_name, options, option_help, NULL);
 	}
 
 	/* Connect to the card */
@@ -437,8 +450,8 @@ main(int argc, char **argv)
 	sc_pkcs15init_set_callbacks(&callbacks);
 
 	/* Bind the card-specific operations and load the profile */
-	if ((r = sc_pkcs15init_bind(card, opt_profile,
-		opt_card_profile, &profile)) < 0) {
+	r = sc_pkcs15init_bind(card, opt_profile, opt_card_profile, NULL, &profile);
+	if (r < 0) {
 		printf("Couldn't bind to the card: %s\n", sc_strerror(r));
 		return 1;
 	}
@@ -471,7 +484,7 @@ main(int argc, char **argv)
 
 				r = sc_pkcs15_bind(card, &aid, &p15card);
 			}
-			else   {	
+			else   {
 				r = sc_pkcs15_bind(card, NULL, &p15card);
 			}
 			if (r) {
@@ -545,6 +558,12 @@ main(int argc, char **argv)
 		case ACTION_SANITY_CHECK:
 			r = do_sanity_check(profile);
 			break;
+		case ACTION_UPDATE_LAST_UPDATE:
+			profile->dirty = 1;
+			break;
+		case ACTION_ERASE_APPLICATION:
+			r = do_erase_application(card, profile);
+			break;
 		default:
 			util_fatal("Action not yet implemented\n");
 		}
@@ -609,7 +628,7 @@ do_assert_pristine(sc_card_t *in_card)
 
 	sc_format_path("3F00", &path);
 	r = sc_select_file(in_card, &path, NULL);
-	if (r) 
+	if (r)
 		goto end;
 
 	sc_format_path("2F00", &path);
@@ -657,7 +676,7 @@ do_erase(sc_card_t *in_card, struct sc_profile *profile)
 		if (sc_hex_to_bin(opt_bind_to_aid, aid.value, &aid.len))   {
 			fprintf(stderr, "Invalid AID value: '%s'\n", opt_bind_to_aid);
 			return 1;
-									                
+
 		}
 
 		r = sc_pkcs15init_erase_card(p15card, profile, &aid);
@@ -671,6 +690,17 @@ do_erase(sc_card_t *in_card, struct sc_profile *profile)
 	return r;
 }
 
+static int
+do_erase_application(sc_card_t *in_card, struct sc_profile *profile)
+{
+	int r;
+
+	ignore_cmdline_pins--;
+	r = do_erase(in_card, profile);
+	ignore_cmdline_pins++;
+	return r;
+}
+
 static int do_finalize_card(sc_card_t *in_card, struct sc_profile *profile)
 {
 	return sc_pkcs15init_finalize_card(in_card, profile);
@@ -714,9 +744,9 @@ do_init_app(struct sc_profile *profile)
 		role = "user";
 	else
 		hints.flags |= SC_UI_PIN_OPTIONAL; /* SO PIN is always optional */
-			
 
-	if ((info.attrs.pin.flags & SC_PKCS15_PIN_FLAG_UNBLOCK_DISABLED) 
+
+	if ((info.attrs.pin.flags & SC_PKCS15_PIN_FLAG_UNBLOCK_DISABLED)
 			&& (info.attrs.pin.flags & SC_PKCS15_PIN_FLAG_SO_PIN))
 		so_puk_disabled = 1;
 
@@ -802,7 +832,7 @@ do_store_pin(struct sc_profile *profile)
 	args.pin_len = strlen(opt_pins[0]);
 	args.label = opt_label;
 
-	if (!(info.attrs.pin.flags & SC_PKCS15_PIN_FLAG_UNBLOCK_DISABLED) 
+	if (!(info.attrs.pin.flags & SC_PKCS15_PIN_FLAG_UNBLOCK_DISABLED)
 			&& opt_pins[1] == NULL) {
 		sc_pkcs15init_get_pin_info(profile, SC_PKCS15INIT_USER_PUK, &info);
 
@@ -846,15 +876,14 @@ do_store_private_key(struct sc_profile *profile)
 	if (ncerts) {
 		char	namebuf[256];
 
-		printf("Importing %d certificates:\n", ncerts);
-		for (i = 0; i < ncerts; i++) {
-			printf("  %d: %s\n",
-				i, X509_NAME_oneline(cert[i]->cert_info->subject,
+		printf("Importing %d certificates:\n", opt_ignore_ca_certs ? 1 : ncerts);
+		for (i = 0; i < ncerts && !(i && opt_ignore_ca_certs); i++)
+			printf("  %d: %s\n", i, X509_NAME_oneline(cert[i]->cert_info->subject,
 					namebuf, sizeof(namebuf)));
-		}
 	}
 
-	if ((r = do_convert_private_key(&args.key, pkey)) < 0)
+	r = sc_pkcs15_convert_prkey(&args.key, pkey);
+	if (r < 0)
 		return r;
 	init_gost_params(&args.params.gost, pkey);
 
@@ -886,6 +915,11 @@ do_store_private_key(struct sc_profile *profile)
 		args.x509_usage = opt_x509_usage? opt_x509_usage : usage;
 	}
 
+	args.access_flags |=
+		  SC_PKCS15_PRKEY_ACCESS_SENSITIVE
+		| SC_PKCS15_PRKEY_ACCESS_ALWAYSSENSITIVE
+		| SC_PKCS15_PRKEY_ACCESS_NEVEREXTRACTABLE;
+
 	r = sc_pkcs15init_store_private_key(p15card, profile, &args, NULL);
 
 	if (r < 0)
@@ -898,6 +932,9 @@ do_store_private_key(struct sc_profile *profile)
 		struct sc_pkcs15init_certargs cargs;
 		char	namebuf[SC_PKCS15_MAX_LABEL_SIZE-1];
 
+		if (i && opt_ignore_ca_certs)
+			break;
+
 		memset(&cargs, 0, sizeof(cargs));
 
 		/* Encode the cert */
@@ -906,8 +943,9 @@ do_store_private_key(struct sc_profile *profile)
 
 		X509_check_purpose(cert[i], -1, -1);
 		cargs.x509_usage = cert[i]->ex_kusage;
-		cargs.label = X509_NAME_oneline(cert[i]->cert_info->subject,
-					namebuf, sizeof(namebuf));
+		cargs.label = cert_common_name(cert[i]);
+		if (!cargs.label)
+			cargs.label = X509_NAME_oneline(cert[i]->cert_info->subject, namebuf, sizeof(namebuf));
 
 		/* Just the first certificate gets the same ID
 		 * as the private key. All others get
@@ -918,23 +956,20 @@ do_store_private_key(struct sc_profile *profile)
 				cargs.label = opt_cert_label;
 		} else {
 			if (is_cacert_already_present(&cargs)) {
-				printf("Certificate #%d already present, "
-					"not stored.\n", i);
+				printf("Certificate #%d already present, not stored.\n", i);
 				goto next_cert;
 			}
 			cargs.authority = 1;
 		}
 
-		r = sc_pkcs15init_store_certificate(p15card, profile,
-			       	&cargs, NULL);
+		r = sc_pkcs15init_store_certificate(p15card, profile, &cargs, NULL);
 next_cert:
 		free(cargs.der_encoded.value);
 	}
-	
+
 	/* No certificates - store the public key */
-	if (ncerts == 0) {
+	if (ncerts == 0)
 		r = do_store_public_key(profile, pkey);
-	}
 
 	return r;
 }
@@ -970,12 +1005,12 @@ is_cacert_already_present(struct sc_pkcs15init_certargs *args)
 		if (r < 0 || !cert)
 			continue;
 
-		if (cert->data_len == args->der_encoded.len
-		     && !memcmp(cert->data, args->der_encoded.value,
-				     cert->data_len)) {
+		if (cert->data.len == args->der_encoded.len
+				&& !memcmp(cert->data.value, args->der_encoded.value, cert->data.len)) {
 			sc_pkcs15_free_certificate(cert);
 			return 1;
 		}
+
 		sc_pkcs15_free_certificate(cert);
 		cert=NULL;
 	}
@@ -997,11 +1032,12 @@ do_store_public_key(struct sc_profile *profile, EVP_PKEY *pkey)
 	if (opt_objectid)
 		sc_pkcs15_format_id(opt_objectid, &args.id);
 	args.label = (opt_pubkey_label != 0 ? opt_pubkey_label : opt_label);
+	args.x509_usage = opt_x509_usage;
 
 	if (pkey == NULL)
 		r = do_read_public_key(opt_infile, opt_format, &pkey);
 	if (r >= 0) {
-		r = do_convert_public_key(&args.key, pkey);
+		r = sc_pkcs15_convert_pubkey(&args.key, pkey);
 		if (r >= 0)
 			init_gost_params(&args.params.gost, pkey);
 	}
@@ -1051,8 +1087,8 @@ do_read_check_certificate(sc_pkcs15_cert_t *sc_oldcert,
 	int r;
 
 	/* Get the public key from the old cert */
-	ptr = sc_oldcert->data;
-	oldcert = d2i_X509(NULL, &ptr, sc_oldcert->data_len);
+	ptr = sc_oldcert->data.value;
+	oldcert = d2i_X509(NULL, &ptr, sc_oldcert->data.len);
 
 	if (oldcert == NULL)
 		return SC_ERROR_INTERNAL;
@@ -1149,12 +1185,12 @@ static int
 do_store_data_object(struct sc_profile *profile)
 {
 	struct sc_pkcs15init_dataargs args;
-	u8	*data;
+	unsigned char *data = NULL;
 	size_t	datalen;
 	int	r=0;
 
 	memset(&args, 0, sizeof(args));
-	args.app_oid.value[0] = -1;
+	sc_init_oid(&args.app_oid);
 
 	if (opt_objectid)
 		sc_pkcs15_format_id(opt_objectid, &args.id);
@@ -1170,10 +1206,11 @@ do_store_data_object(struct sc_profile *profile)
 		/* der_encoded contains the plain data, nothing DER encoded */
 		args.der_encoded.value = data;
 		args.der_encoded.len = datalen;
-		r = sc_pkcs15init_store_data_object(p15card, profile,
-					&args, NULL);
+		r = sc_pkcs15init_store_data_object(p15card, profile, &args, NULL);
 	}
 
+	if (data)
+		free(data);
 	return r;
 }
 
@@ -1417,7 +1454,7 @@ do_change_attributes(struct sc_profile *profile, unsigned int myopt_type)
 	if (opt_label != NULL) {
 		strlcpy(obj->label, opt_label, sizeof(obj->label));
 	}
-	
+
 	r = sc_pkcs15init_update_any_df(p15card, profile, obj->df, 0);
 
 	return r;
@@ -1438,9 +1475,9 @@ do_generate_key(struct sc_profile *profile, const char *spec)
 
 	if ((r = init_keyargs(&keygen_args.prkey_args)) < 0)
 		return r;
-        keygen_args.prkey_args.access_flags |= 
-		  SC_PKCS15_PRKEY_ACCESS_SENSITIVE 
-		| SC_PKCS15_PRKEY_ACCESS_ALWAYSSENSITIVE 
+        keygen_args.prkey_args.access_flags |=
+		  SC_PKCS15_PRKEY_ACCESS_SENSITIVE
+		| SC_PKCS15_PRKEY_ACCESS_ALWAYSSENSITIVE
 		| SC_PKCS15_PRKEY_ACCESS_NEVEREXTRACTABLE
 		| SC_PKCS15_PRKEY_ACCESS_LOCAL;
 
@@ -1638,7 +1675,7 @@ get_pin_callback(struct sc_profile *profile,
 	int	allocated = 0;
 
 	if (info->auth_type != SC_PKCS15_PIN_AUTH_TYPE_PIN)
-		return SC_ERROR_NOT_SUPPORTED; 
+		return SC_ERROR_NOT_SUPPORTED;
 
 	if (label)
 		snprintf(namebuf, sizeof(namebuf), "PIN [%s]", label);
@@ -1650,19 +1687,19 @@ get_pin_callback(struct sc_profile *profile,
 			switch (id) {
 			case SC_PKCS15INIT_USER_PIN:
 				name = "User PIN";
-				secret = opt_pins[OPT_PIN1 & 3]; 
+				secret = opt_pins[OPT_PIN1 & 3];
 				break;
 			case SC_PKCS15INIT_USER_PUK:
 				name = "User PIN unlock key";
-				secret = opt_pins[OPT_PUK1 & 3]; 
+				secret = opt_pins[OPT_PUK1 & 3];
 				break;
 			case SC_PKCS15INIT_SO_PIN:
 				name = "Security officer PIN";
-				secret = opt_pins[OPT_PIN2 & 3]; 
+				secret = opt_pins[OPT_PIN2 & 3];
 				break;
 			case SC_PKCS15INIT_SO_PUK:
 				name = "Security officer PIN unlock key";
-				secret = opt_pins[OPT_PUK2 & 3]; 
+				secret = opt_pins[OPT_PUK2 & 3];
 				break;
 			}
 		}
@@ -1753,7 +1790,7 @@ get_pin_callback(struct sc_profile *profile,
 }
 
 
-static int 
+static int
 get_key_callback(struct sc_profile *profile,
 			int method, int reference,
 			const u8 *def_key, size_t def_key_size,
@@ -2002,7 +2039,7 @@ do_read_pem_public_key(const char *filename)
 		util_fatal("Unable to open %s: %m", filename);
 	pk = PEM_read_bio_PUBKEY(bio, NULL, NULL, NULL);
 	BIO_free(bio);
-	if (pk == NULL) 
+	if (pk == NULL)
 		ossl_print_errors();
 	return pk;
 }
@@ -2018,7 +2055,7 @@ do_read_der_public_key(const char *filename)
 		util_fatal("Unable to open %s: %m", filename);
 	pk = d2i_PUBKEY_bio(bio, NULL);
 	BIO_free(bio);
-	if (pk == NULL) 
+	if (pk == NULL)
 		ossl_print_errors();
 	return pk;
 }
@@ -2055,7 +2092,7 @@ do_read_pem_certificate(const char *filename)
 		util_fatal("Unable to open %s: %m", filename);
 	xp = PEM_read_bio_X509(bio, NULL, NULL, NULL);
 	BIO_free(bio);
-	if (xp == NULL) 
+	if (xp == NULL)
 		ossl_print_errors();
 	return xp;
 }
@@ -2071,7 +2108,7 @@ do_read_der_certificate(const char *filename)
 		util_fatal("Unable to open %s: %m", filename);
 	xp = d2i_X509_bio(bio, NULL);
 	BIO_free(bio);
-	if (xp == NULL) 
+	if (xp == NULL)
 		ossl_print_errors();
 	return xp;
 }
@@ -2094,18 +2131,22 @@ do_read_certificate(const char *name, const char *format, X509 **out)
 	return 0;
 }
 
-static int determine_filesize(const char *filename) {
+static size_t determine_filesize(const char *filename)
+{
 	FILE *fp;
-	size_t size;
+	long ll;
+
+	if ((fp = fopen(filename,"rb")) == NULL)
+		util_fatal("Unable to open %s: %m", filename);
 
-	if ((fp = fopen(filename,"rb")) == NULL) {
-	  util_fatal("Unable to open %s: %m", filename);
-	  }
 	fseek(fp,0L,SEEK_END);
-	size = ftell(fp);
+	ll = ftell(fp);
+	if (ll == -1l)
+		util_fatal("fseek/ftell error");
+
 	fclose(fp);
-	return size;
-	}
+	return (size_t)ll;
+}
 
 static int
 do_read_data_object(const char *name, u8 **out, size_t *outlen)
@@ -2115,10 +2156,10 @@ do_read_data_object(const char *name, u8 **out, size_t *outlen)
 	int c;
 
 	*out = malloc(filesize);
-	if (*out == NULL) {
+	if (*out == NULL)
 		return SC_ERROR_OUT_OF_MEMORY;
-	}
- 
+
+
         inf = fopen(name, "rb");
         if (inf == NULL) {
                 fprintf(stderr, "Unable to open '%s' for reading.\n", name);
@@ -2135,141 +2176,39 @@ do_read_data_object(const char *name, u8 **out, size_t *outlen)
 	return 0;
 }
 
-static int
-do_convert_bignum(sc_pkcs15_bignum_t *dst, const BIGNUM *src)
+static char *
+cert_common_name(X509 *x509)
 {
-	if (src == 0)
-		return 0;
-	dst->len = BN_num_bytes(src);
-	dst->data = malloc(dst->len);
-	BN_bn2bin(src, dst->data);
-	return 1;
+	X509_NAME_ENTRY *ne = NULL;
+	ASN1_STRING *a_str = NULL;
+	char *out = NULL;
+	unsigned char *tmp = NULL;
+	int idx, out_len = 0;
+
+	idx = X509_NAME_get_index_by_NID(X509_get_subject_name(x509), NID_commonName, -1);
+	if (idx < 0)
+		return NULL;
+
+	ne = X509_NAME_get_entry(X509_get_subject_name(x509), idx);
+	if (!ne)
+	       return NULL;
+
+	a_str = X509_NAME_ENTRY_get_data(ne);
+	if (!a_str)
+		return NULL;
+
+	out_len = ASN1_STRING_to_UTF8(&tmp, a_str);
+	if (!tmp)
+		return NULL;
+
+	out = calloc(1, out_len + 1);
+	if (out)
+		memcpy(out, tmp, out_len);
+	OPENSSL_free(tmp);
+
+	return out;
 }
 
-static int do_convert_private_key(struct sc_pkcs15_prkey *key, EVP_PKEY *pk)
-{
-	switch (pk->type) {
-	case EVP_PKEY_RSA: {
-		struct sc_pkcs15_prkey_rsa *dst = &key->u.rsa;
-		RSA *src = EVP_PKEY_get1_RSA(pk);
-
-		key->algorithm = SC_ALGORITHM_RSA;
-		if (!do_convert_bignum(&dst->modulus, src->n)
-		 || !do_convert_bignum(&dst->exponent, src->e)
-		 || !do_convert_bignum(&dst->d, src->d)
-		 || !do_convert_bignum(&dst->p, src->p)
-		 || !do_convert_bignum(&dst->q, src->q))
-			util_fatal("Invalid/incomplete RSA key.\n");
-		if (src->iqmp && src->dmp1 && src->dmq1) {
-			do_convert_bignum(&dst->iqmp, src->iqmp);
-			do_convert_bignum(&dst->dmp1, src->dmp1);
-			do_convert_bignum(&dst->dmq1, src->dmq1);
-		}
-		RSA_free(src);
-		break;
-		}
-	case EVP_PKEY_DSA: {
-		struct sc_pkcs15_prkey_dsa *dst = &key->u.dsa;
-		DSA *src = EVP_PKEY_get1_DSA(pk);
-
-		key->algorithm = SC_ALGORITHM_DSA;
-		do_convert_bignum(&dst->pub, src->pub_key);
-		do_convert_bignum(&dst->p, src->p);
-		do_convert_bignum(&dst->q, src->q);
-		do_convert_bignum(&dst->g, src->g);
-		do_convert_bignum(&dst->priv, src->priv_key);
-		DSA_free(src);
-		break;
-		}
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L && !defined(OPENSSL_NO_EC)
-	case NID_id_GostR3410_2001: {
-		struct sc_pkcs15_prkey_gostr3410 *dst = &key->u.gostr3410;
-		EC_KEY *src = EVP_PKEY_get0(pk);
-
-		assert(src);
-		key->algorithm = SC_ALGORITHM_GOSTR3410;
-		assert(EC_KEY_get0_private_key(src));
-		do_convert_bignum(&dst->d, EC_KEY_get0_private_key(src));
-		break;
-		}
-#endif /* OPENSSL_VERSION_NUMBER >= 0x10000000L && !defined(OPENSSL_NO_EC) */
-	default:
-		util_fatal("Unsupported key algorithm\n");
-	}
-
-	return 0;
-}
-
-static int do_convert_public_key(struct sc_pkcs15_pubkey *key, EVP_PKEY *pk)
-{
-	switch (pk->type) {
-	case EVP_PKEY_RSA: {
-		struct sc_pkcs15_pubkey_rsa *dst = &key->u.rsa;
-		RSA *src = EVP_PKEY_get1_RSA(pk);
-
-		key->algorithm = SC_ALGORITHM_RSA;
-		if (!do_convert_bignum(&dst->modulus, src->n)
-		 || !do_convert_bignum(&dst->exponent, src->e))
-			util_fatal("Invalid/incomplete RSA key.\n");
-		RSA_free(src);
-		break;
-		}
-	case EVP_PKEY_DSA: {
-		struct sc_pkcs15_pubkey_dsa *dst = &key->u.dsa;
-		DSA *src = EVP_PKEY_get1_DSA(pk);
-
-		key->algorithm = SC_ALGORITHM_DSA;
-		do_convert_bignum(&dst->pub, src->pub_key);
-		do_convert_bignum(&dst->p, src->p);
-		do_convert_bignum(&dst->q, src->q);
-		do_convert_bignum(&dst->g, src->g);
-		DSA_free(src);
-		break;
-		}
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L && !defined(OPENSSL_NO_EC)
-	case NID_id_GostR3410_2001: {
-		struct sc_pkcs15_pubkey_gostr3410 *dst = &key->u.gostr3410;
-		EC_KEY *eckey = EVP_PKEY_get0(pk);
-		const EC_POINT *point;
-		BIGNUM *X, *Y;
-		int r = 0;
-
-		assert(eckey);
-		point = EC_KEY_get0_public_key(eckey);
-		if (!point)
-			return SC_ERROR_INTERNAL;
-		X = BN_new();
-		Y = BN_new();
-		if (X && Y && EC_KEY_get0_group(eckey))
-			r = EC_POINT_get_affine_coordinates_GFp(EC_KEY_get0_group(eckey),
-					point, X, Y, NULL);
-		if (r == 1) {
-			dst->xy.len = BN_num_bytes(X) + BN_num_bytes(Y);
-			dst->xy.data = malloc(dst->xy.len);
-			if (dst->xy.data) {
-				BN_bn2bin(Y, dst->xy.data);
-				BN_bn2bin(X, dst->xy.data + BN_num_bytes(Y));
-				r = sc_mem_reverse(dst->xy.data, dst->xy.len);
-				if (!r)
-					r = 1;
-				key->algorithm = SC_ALGORITHM_GOSTR3410;
-			}
-			else
-				r = -1;
-		}
-		BN_free(X);
-		BN_free(Y);
-		if (r != 1)
-			return SC_ERROR_INTERNAL;
-		break;
-		}
-#endif /* OPENSSL_VERSION_NUMBER >= 0x10000000L && !defined(OPENSSL_NO_EC) */
-	default:
-		util_fatal("Unsupported key algorithm\n");
-	}
-
-	return 0;
-}
 
 static int do_convert_cert(sc_pkcs15_der_t *der, X509 *cert)
 {
@@ -2299,7 +2238,7 @@ parse_objects(const char *list, unsigned int action)
 
 	while (1) {
 		int	len, n;
-		
+
 		while (*list == ',')
 			list++;
 		if (!*list)
@@ -2369,7 +2308,7 @@ parse_x509_usage(const char *list, unsigned int *res)
 
 	while (1) {
 		int	len, n, match = 0;
-		
+
 		while (*list == ',')
 			list++;
 		if (!*list)
@@ -2471,7 +2410,7 @@ handle_option(const struct option *opt)
 		opt_format = optarg;
 		break;
 	case 'h':
-		util_print_usage_and_die(app_name, options, option_help);
+		util_print_usage_and_die(app_name, options, option_help, NULL);
 	case 'i':
 		opt_objectid = optarg;
 		break;
@@ -2568,8 +2507,18 @@ handle_option(const struct option *opt)
 	case OPT_SANITY_CHECK:
 		this_action = ACTION_SANITY_CHECK;
 		break;
+	case OPT_UPDATE_LAST_UPDATE:
+		this_action = ACTION_UPDATE_LAST_UPDATE;
+		break;
+	case OPT_ERASE_APPLICATION:
+		opt_bind_to_aid = optarg;
+		this_action = ACTION_ERASE_APPLICATION;
+		break;
+	case OPT_IGNORE_CA_CERTIFICATES:
+		opt_ignore_ca_certs = 1;
+		break;
 	default:
-		util_print_usage_and_die(app_name, options, option_help);
+		util_print_usage_and_die(app_name, options, option_help, NULL);
 	}
 
 	if ((opt_actions & (1 << this_action)) && opt->has_arg != no_argument) {
@@ -2579,7 +2528,7 @@ handle_option(const struct option *opt)
 		if (isprint(opt->val))
 			fprintf(stderr, " -%c", opt->val);
 		fprintf(stderr, " more than once.\n");
-		util_print_usage_and_die(app_name, options, option_help);
+		util_print_usage_and_die(app_name, options, option_help, NULL);
 	}
 	if (this_action)
 		opt_actions |= (1 << this_action);
@@ -2588,13 +2537,13 @@ handle_option(const struct option *opt)
 		fprintf(stderr, "Error: "
 		"The --no-so-pin option and --so-pin/--so-puk are mutually"
 		"exclusive.\n");
-		util_print_usage_and_die(app_name, options, option_help);
+		util_print_usage_and_die(app_name, options, option_help, NULL);
 	}
 
 	if ((opt_actions & (1 << ACTION_ERASE)) &&
 		(opt_actions != (1 << ACTION_ERASE))) {
 		fprintf(stderr, "Error: erasing a card is incompatible with all other actions\n");
-		util_print_usage_and_die(app_name, options, option_help);
+		util_print_usage_and_die(app_name, options, option_help, NULL);
 	}
 }
 
@@ -2668,7 +2617,7 @@ read_options_file(const char *filename)
 					break;
 			if (!o->name) {
 				util_error("Unknown option \"%s\"\n", name);
-				util_print_usage_and_die(app_name, options, option_help);
+				util_print_usage_and_die(app_name, options, option_help, NULL);
 			}
 			if (o->has_arg != no_argument) {
 				optarg = strtok(NULL, "");
@@ -2681,7 +2630,7 @@ read_options_file(const char *filename)
 			if (o->has_arg == required_argument
 			 && (!optarg || !*optarg)) {
 				util_error("Option %s: missing argument\n", name);
-				util_print_usage_and_die(app_name, options, option_help);
+				util_print_usage_and_die(app_name, options, option_help, NULL);
 			}
 			handle_option(o);
 			name = strtok(NULL, " \t");
@@ -2714,7 +2663,7 @@ ossl_print_errors(void)
  *
  * @hints	dialog hints
  * @out		PIN entered by the user; must be freed.
- * 		NULL if dialog was canceled.
+ *		NULL if dialog was canceled.
  */
 int get_pin(sc_ui_hints_t *hints, char **out)
 {
@@ -2723,17 +2672,16 @@ int get_pin(sc_ui_hints_t *hints, char **out)
 	int		flags = hints->flags;
 
 	pin_info = hints->info.pin;
-	if (pin_info && pin_info->auth_type == SC_PKCS15_PIN_AUTH_TYPE_PIN)
+	if (pin_info && (pin_info->auth_type != SC_PKCS15_PIN_AUTH_TYPE_PIN))
 		return SC_ERROR_NOT_SUPPORTED;
 
 	if (!(label = hints->obj_label)) {
-		if (pin_info == NULL) {
+		if (pin_info == NULL)
 			label = "PIN";
-		} else if (pin_info->attrs.pin.flags & SC_PKCS15_PIN_FLAG_SO_PIN) {
+		else if (pin_info->attrs.pin.flags & SC_PKCS15_PIN_FLAG_SO_PIN)
 			label = "Security Officer PIN";
-		} else {
+		else
 			label = "User PIN";
-		}
 	}
 
 	if (hints->p15card) {
@@ -2823,7 +2771,7 @@ static int verify_pin(struct sc_pkcs15_card *p15card, char *auth_id_str)
 	if (!auth_id_str)   {
 	        struct sc_pkcs15_object *objs[32];
         	int ii;
-		
+
 		r = sc_pkcs15_get_objects(p15card, SC_PKCS15_TYPE_AUTH_PIN, objs, 32);
 		if (r < 0) {
                         fprintf(stderr, "PIN code enumeration failed: %s\n", sc_strerror(r));
@@ -2871,7 +2819,7 @@ static int verify_pin(struct sc_pkcs15_card *p15card, char *auth_id_str)
 
 		if (pin_obj->label)
 			snprintf(pin_label, sizeof(pin_label), "User PIN [%s]", pin_obj->label);
-		else 
+		else
 			snprintf(pin_label, sizeof(pin_label), "User PIN");
                 memset(&hints, 0, sizeof(hints));
                 hints.dialog_name = "pkcs15init.get_pin";
